@@ -1,6 +1,6 @@
 // ExMgr.h
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <signal.h>
+#include <sys/wait.h>
+#include <glob.h>
 #include <mlcd.h>
 
 /* VxWorks coding conventions. */
@@ -40,11 +43,12 @@
 typedef int STATUS;
 
 /* Miscellaneous macros. */
-#define cMaxWriteIrps 8			/* TODO: Adjust? */
+#define cMaxWriteIrps 8
 #define IN_PROGRESS (OK+1)
 #define SETTHIS(x) (this->x=x)
 #define DEBUG_STRING(s) (0)
-#define dmSetUint32(key,value) /* OK */
+#define dmSetUint32(key,value) pMgr->setTransportMode(value)
+enum { bpUsbCurrentModeUnknown,bpUsbCurrentModeMlc,bpUsbCurrentModeDot4 };
 #define nettestRegister(args...)
 
 /* Logging. */
@@ -202,7 +206,7 @@ class ExMsg: public QueueEntry {
 
 class ExMsgQueue: public Queue {
     public:
-	void add(ExMsg *pMsg) { base_add(pMsg); }
+	void add(ExMsg *pMsg,ExMsg *before=0) { base_add(pMsg,before); }
 	ExMsg *peek(ExMsg *pMsg=0) { return (ExMsg *)base_peek(pMsg); }
 	ExMsg *pop(ExMsg *pMsg=0) { return (ExMsg *)base_pop(pMsg); }
 };
@@ -237,17 +241,50 @@ class PolledTimer {
 		pActual->tv_usec-=start.tv_usec;
 		pActual->tv_sec-=start.tv_sec;
 	}
-	static int compareTimes(struct timeval *a,struct timeval *b);
+	static int compareTimes(struct timeval *a,struct timeval *b) {
+		if (a->tv_sec<b->tv_sec) {
+	    lt:		return -1;
+		}
+		if (a->tv_sec>b->tv_sec) {
+	    gt:		return 1;
+		}
+		if (a->tv_usec<b->tv_usec) goto lt;
+		if (a->tv_usec>b->tv_usec) goto gt;
+		return 0;
+	}
 
-	void setTimeout(struct timeval *pTimeout);
-	int isTimedOut(void);
-#if 0
-	void getTimeoutTime(struct timeval *pTime) {
-		pTime->tv_sec=end.tv_sec;
-		pTime->tv_usec=end.tv_usec;
+	void setTimeout(struct timeval *pTimeout) {
+		static const int m=1000000;
+
+		getTime(&start);
+		end.tv_sec=start.tv_sec+pTimeout->tv_sec;
+		end.tv_usec=start.tv_usec+pTimeout->tv_usec;
+		end.tv_sec+=end.tv_usec/m;
+		end.tv_usec%=m;
+	}
+	int isTimedOut(void) {
+		struct timeval time;
+		getTime(&time);
+		if (compareTimes(&start,&end)<0) {
+			return (compareTimes(&time,&start)<0 ||
+				compareTimes(&time,&end)>=0);
+		}
+			return (compareTimes(&time,&start)<0 &&
+				compareTimes(&time,&end)>=0);
+	}
+	void getRemainingTime(struct timeval *pTime) {
+		struct timeval time;
+		getTime(&time);
+
+		pTime->tv_sec=pTime->tv_usec=0;
+		if (!isTimedOut()) {
+			/* This is a quick-and dirty substitute for proper
+			 * borrow-subtraction. */
+			if (!(pTime->tv_sec=end.tv_sec-time.tv_sec)) {
+				pTime->tv_usec=end.tv_usec-time.tv_usec;
+			}
+		}
 	}
-#endif
-	void getRemainingTime(struct timeval *pTime);
 };
 
 class ExWatchdogTimer: public QueueEntry {
@@ -417,6 +454,7 @@ class ExBdr: public QueueEntry {
 	void setTCD(TCD tcd) { SETTHIS(tcd); }
 	int getTransportHeaderFlags(void) { return 0; }
 	void setTransportHeaderFlags(int flags) { }
+	void setIsFirstDataBdr(int foo) { }
     protected:
 	void reset(void) {
 		setNext(0);
@@ -434,7 +472,7 @@ inline void exReturnPhysicalPortBuffer(ExBdr *pBdr) {
 class ExBdrQueue: public Queue {
     public:
 	ExBdrQueue(char *name=0): Queue(name) { }
-	void add(ExBdr *pBdr) { base_add(pBdr); }
+	void add(ExBdr *pBdr,ExBdr *before=0) { base_add(pBdr,before); }
 	void Add(ExBdr *pBdr) { add(pBdr); }
 	ExBdr *peek(void) { return (ExBdr *)base_peek(); }
 	ExBdr *Peek(void) { return peek(); }
@@ -470,7 +508,7 @@ class ExBufferPool {
 #endif
 };
 
-#define EX_KNOB_TRANSPORT 0x300
+#define EX_KNOB_TRANSPORT 0x3000
 #include "transport/ExTransportKnobs.h"
 
 STATUS tknobGetWorkingValue(int port,int id,int *pValue);
@@ -479,8 +517,9 @@ STATUS exTknobGetInit(int id,int *pValue);
 class ExMgr: public ExMsgHandler {
 	// Major helper classes:
     protected:
+	static const int FORWARD_DATA_TIMEOUT=5;
+	static const int COMMAND_REPLY_TIMEOUT=5;
 	ExBufferPool *pBufferPool;
-	ExTransport *pTransport;
 
 	// Constructor, destructor, dump:
     public:
@@ -492,12 +531,24 @@ class ExMgr: public ExMsgHandler {
 
 	// Runtime flow of control:
     protected:
+	enum ExInterface {
+		EX_INTERFACE_NONE=-1,
+		EX_INTERFACE_MLC=0,
+		EX_INTERFACE_PRINT,
+		EX_INTERFACE_COUNT
+	};
+	char *getInterfaceString(int iface) {
+		if (iface==EX_INTERFACE_MLC) return "MLC";
+		if (iface==EX_INTERFACE_PRINT) return "print";
+		return "???";
+	}
 	static const int port=0;
 	int noFork;
-	int stayActivated;
+	int noDrain;
+	int forceMlcDot4;
 	int noDot4;
+	int noMlc;
 	int noPmlMultiplexing;
-	int sleepBeforeOpen;
 	int initialized;
 	fd_set rset;
 	fd_set wset;
@@ -509,18 +560,17 @@ class ExMgr: public ExMsgHandler {
 		EX_CONTEXT_LLIO_SERVICE_OUTPUT,
 		EX_CONTEXT_LLIO_SERVICE_INPUT
 	} exContext;
-	enum {
+	enum ExState {
 		EX_STATE_DOWN=0,
 		EX_STATE_ACTIVATING,
-		EX_STATE_UP,
-		EX_STATE_DEACTIVATING
+		EX_STATE_UP
 	} exState;
 	int exActivateCount;
 	int exCloseCount;
 	int exCloseReason;
-	int exReactivateFlag;
 	int tryDot4;
 	int tryMlc;
+	int transportMode;
 	int miser;
     public:
 	int isInitialized(void) { return initialized; } 
@@ -532,19 +582,68 @@ class ExMgr: public ExMsgHandler {
 	STATUS _fdRegister(int fd,int r,int w,char *file,int line);
 #define fdRegister(fd,r,w) _fdRegister(fd,r,w,__FILE__,__LINE__)
 	int fdIsRegisteredForRead(int fd) { return FD_ISSET(fd,&rset); }
+	void fdInit(int *fd) {
+		*fd=ERROR;
+	}
+	void fdClose(int *fd) {
+		if (*fd>=0) {		/* !=ERROR */
+			fdRegister(*fd,-1,-1);
+			close(*fd);
+			fdInit(fd);
+		}
+	}
+	int fdIsReadable(int fd) {
+		fd_set rset;
+		struct timeval timeout={0,0};
+		FD_ZERO(&rset);
+		FD_SET(fd,&rset);
+		return select(fd+1,&rset,0,0,&timeout);
+	}
+	void pidInit(pid_t *pid) {
+		*pid=ERROR;
+	}
+	void pidKill(pid_t *pid) {
+		if (*pid>1) {
+			kill(*pid,SIGTERM);
+			waitpid(*pid,0,0);
+		}
+		pidInit(pid);
+	}
+	int pidHasExited(pid_t *pid) {
+		if (*pid<1) return ERROR;
+		pid_t r=waitpid(*pid,0,WNOHANG);
+		if (!r) return 0;
+		r=(r==*pid);
+		pidInit(pid);
+		if (r) return 1;
+		return ERROR;
+	}
     public:
 	int getPortNumber(void) { return port; }
 	int getBufferPoolMgr(void) { return 0; }
     protected:
 	STATUS exActivate(void);
+	STATUS activateOneTransport(ExInterface iface);
+	STATUS activateOneTransport(void);
 	void parseDeviceID(void);
-	virtual int getDefaultTryDot4(void) { return 1; }
+	virtual int getDefaultTryMlcDot4(void) { return forceMlcDot4; }
     public:
 	STATUS tknobGet(int id,int *pValue);
+	void setTransportMode(int value) { transportMode=value; }
     protected:
+	char *getTransportModeString(void) {
+		if (transportMode==bpUsbCurrentModeMlc) {
+			return "MLC";
+		}
+		if (transportMode==bpUsbCurrentModeDot4) {
+			return "1284.4";
+		}
+		return "print-only";
+	}
 	void exActivateComplete(void) {
 		exState=EX_STATE_UP;
-		LOG_SYSLOG(cEXBP,0,"ptal-mlcd successfully activated.\n");
+		LOG_SYSLOG(cEXBP,0,"ptal-mlcd successfully activated, "
+			"mode=%s.\n",getTransportModeString());
 	}
     public:
 	void exClose(int reason=0);
@@ -647,15 +746,16 @@ class ExMgr: public ExMsgHandler {
 	void consolePostSelect(fd_set *rset,fd_set *wset) {
 		if (FD_ISSET(CONSOLE_STDIN,rset)) consoleService();
 	}
+	int consoleFdSetBlocking(int fd) {
+		if (!consoleAllowRemote) return ERROR;
+		return fdSetBlocking(fd);
+	}
 
 	// Socket management:
     protected:
-	static const char socketPrefix[]="/dev/ptal-mlcd/";
 	static const int SOCKET_BACKLOG=5;
 	char *socketSuffix;
-	char *socketAliasSuffix;
 	char *socketName;
-	char *socketAliasName;
 	int socketFd;
     public:
 	char *getSocketSuffix(void) { return socketSuffix; }
@@ -670,6 +770,8 @@ class ExMgr: public ExMsgHandler {
 
 	// Session management:
     protected:
+	static const int SOCKET_CLOSE_WAIT_TIMEOUT=2;	// seconds.
+	static const int SOCKET_CLOSE_WAIT_DELAY=100000; // usec.
 	static const int MAX_REMOTE_SOCKETS=32;
 	static const int MAX_COMMAND_SESSIONS   =8;
 	static const int MAX_TRANSPORT_SESSIONS =16;
@@ -687,18 +789,25 @@ class ExMgr: public ExMsgHandler {
 	static const int BUFFER_SIZE=(DEFAULT_MAX_DATALEN+MLC_LEN_HEADER);
 	static const int PML_MAX_FORWARD_DATALEN=DEFAULT_MAX_DATALEN;
 	static const int PML_MAX_REVERSE_DATALEN=DEFAULT_MAX_DATALEN;
-	static const int MAX_FORWARD_BUFFERS=50;		/* TODO: Adjust? */
-	static const int SESSION_MIN_BUFFERS=10;		/* TODO: Adjust? */
-	static const int SESSION_BENEFIT_OF_MORE_BUFFERS=1;	/* TODO: Adjust? */
-	static const int EXTRA_REVERSE_BUFFERS=20;		/* TODO: Adjust? */
+	static const int MAX_FORWARD_BUFFERS=50;
+	static const int SESSION_MIN_BUFFERS=20;
+	static const int SESSION_BENEFIT_OF_MORE_BUFFERS=1;
+	static const int EXTRA_REVERSE_BUFFERS=20;
 	static const int MAX_REVERSE_BUFFERS=((MAX_TRANSPORT_SESSIONS*SESSION_MIN_BUFFERS)+EXTRA_REVERSE_BUFFERS);
 	static const int MAX_BUFFERS=(MAX_FORWARD_BUFFERS+MAX_REVERSE_BUFFERS);
-	static const int MAX_BDRS_PER_TRANSACTION=4;		/* TODO: Adjust? */
+	static const int MAX_BDRS_PER_TRANSACTION=4;
+	static const int FIRST_TRANSPORT_SOCKET=1;
+	static const int LAST_TRANSPORT_SOCKET=255;
 	static const int SOCKET_CONSOLE=-1;
 	static const int SOCKET_PID=-2;
 	static const int SOCKET_CMDLINE=-3;
 	static const int SOCKET_DEVNODE=-4;
 	static const int SOCKET_DUMP=-5;
+	static const int SOCKET_RESERVE_SCANNER=-6;
+	static const int SOCKET_RESERVE_FAX_SEND=-7;
+	static const int SOCKET_RESERVE_FAX_RECEIVE=-8;
+	static const int SOCKET_COMPOSITE_PRINT=-9;
+	static const int SOCKET_GLOB_DEVNODES=-10;
 	enum SessionState {
 		SESSION_STATE_SHUTDOWN=-2,
 		SESSION_STATE_STARTUP,		// -1
@@ -724,6 +833,7 @@ class ExMgr: public ExMsgHandler {
 		TCD tcd;
 		ExBdr *pCommandBdr;
 		int pmlTrapsRegistered;
+		int bitbucketSocket;
 	} session[MAX_SESSIONS];
     protected:
 #ifdef JD_DEBUGLITE
@@ -765,14 +875,14 @@ class ExMgr: public ExMsgHandler {
 	STATUS sessionTryLocalLookup(SCD scd);
 	void sessionActivate(void);
 	void sessionInitialRequestsComplete(void);
-	void sessionActivateResponseFromTransport(void);
 	void sessionActivateResponse(void);
 	void sessionLookupResponse(SCD scd,ExSessionLookup *pLookup);
 	void sessionSetRemoteSocketResponse(SCD scd,int status);
 	void sessionOpenChannelResponse(SCD scd,
 		int maxForwardDatalen,int maxReverseDatalen,int status);
-	void sessionStartClose(SCD scd);
-	STATUS sessionFlushClose(SCD scd);
+	void sessionForceClose(void);
+	void sessionStartClose(SCD scd,int flushForwardData=0);
+	STATUS sessionFlushClose(SCD scd,int flushForwardData=0);
 	void sessionCloseChannelResponse(SCD scd,int status);
 	void sessionHandleForwardData(SCD scd);
     public:
@@ -781,44 +891,57 @@ class ExMgr: public ExMsgHandler {
 	void reverseDataReceived(SCD scd,ExBdr *pBdr,int status);
     protected:
 	void sessionServiceOutput(SCD scd);
-	int sessionWrite(SCD scd,unsigned char *data,int datalen);
+	int sessionWrite(SCD scd,const unsigned char *data,int datalen);
 	void sessionDeactivate(void);
 
 	// Low-level I/O (LLIO) (partially handled by derived class):
     protected:
-	static const int LLIO_MAX_POSSIBLE_NAMES=20;	/* TODO: Adjust? */
-	static const int LLIO_MAX_DEVICE_ID_MATCHES=5;	/* TODO: Adjust? */
+	static const int LLIO_MAX_POSSIBLE_NAMES=20;
+	static const int LLIO_MAX_DEVICE_ID_MATCHES=5;
 	static const int MAX_TRANSACTIONS=(MAX_TRANSPORT_SESSIONS)*(cMaxWriteIrps);
+	static const int LLIO_OPEN_TIMEOUT=2;
 	static const int LLIO_READ_TIMEOUT=1;
 	static const int LLIO_READ_LOOP_TIMEOUT=5;
-	enum LlioPollEvent {
-		LLIO_POLL_EVENT_OPEN,
-		LLIO_POLL_EVENT_CLOSE,
-		LLIO_POLL_EVENT_HIT,
-		LLIO_POLL_EVENT_MISS
-	};      //                       1dcm..u
-	static const int LLIO_POLL_RATE_HIGH=     50000;
+	static const int LLIO_WRITE_TIMEOUT=5;
+	static const int LLIO_MLC_BUFFER_LEN=65536;
+	static const int LLIO_PRINT_BUFFER_LEN=DEFAULT_MAX_DATALEN;
+	static const int LLIO_CHANNEL_PRINT=0;
+	static const int LLIO_CHANNEL_MLC_PACKET=77;
+	static const int LLIO_CHANNEL_MLC_RESET=78;
+	static const int LLIO_PRINT_SUBPROCESS_POLL_RATE=5;
+	//                                      1dcm..u
+	static const int LLIO_CH78_RESET_DELAY=  100000;
+	static const int LLIO_DRAIN_TIMEOUT=     250000;
+	static const int LLIO_CLOSE_DELAY=       500000;
+	static const int LLIO_POLL_RATE_HIGH=      1000;
 	static const int LLIO_POLL_RATE_MEDIUM=  100000;
 	static const int LLIO_POLL_RATE_LOW=     500000;
 	static const int LLIO_POLL_THRESHOLD_MEDIUM=5;
 	static const int LLIO_POLL_THRESHOLD_LOW=10;
-	ExWatchdogTimer *llioPollTimer;
 	char *llioPossibleName[LLIO_MAX_POSSIBLE_NAMES];
 	int llioPossibleNameCount;
 	char *llioName;
-	char *llioOverrideDeviceID;
+	glob_t llioGlobBuffer;
+	int llioGlobFlags;
 	char *llioMatchDeviceID[LLIO_MAX_DEVICE_ID_MATCHES];
 	int llioMatchDeviceIDCount;
-	int llioFd;
-	int llioDummyFd;
-	enum LlioPollState {
-		LLIO_POLL_STATE_HIGH,
-		LLIO_POLL_STATE_MEDIUM,
-		LLIO_POLL_STATE_LOW
-	} llioPollState;
-	time_t llioLastHitTime;
-	ExBdrQueue *llioForwardBdrQueue;
-	unsigned char *llioDeviceID;
+	struct {
+		ExTransport *pTransport;
+		ExBdrQueue *pForwardBdrQueue;
+		int active;
+		ExState transportState;
+		struct {
+			int read;
+			int write;
+		} fdPipe;
+		int fdDevice;
+		int overrideMaxForwardPacketSize;
+		pid_t pid;
+	} llioInterface[EX_INTERFACE_COUNT];
+	int llioEnablePrintInterface;
+	ExWatchdogTimer *llioPrintSubprocessWaitTimer;
+	char *llioDeviceID;
+	char *llioSavedDeviceID;
     protected:
 #ifdef JD_DEBUGLITE
 	virtual void llioDump(void);
@@ -829,25 +952,134 @@ class ExMgr: public ExMsgHandler {
 	virtual void llioInit(void) { }
 	void llioDone(void);
 	STATUS llioOpen(void);
-	STATUS llioOpenOne(int *pOpened=0);
-	char *llioGetDeviceNode(void) { return llioName; }
-	virtual unsigned char *llioGetDeviceID(void)=0;
-	virtual STATUS llioSetup(void) { return OK; }
+	void llioGlobInit(void) {
+		llioGlobBuffer.gl_pathc=0;
+		llioGlobBuffer.gl_pathv=0;
+		llioGlobBuffer.gl_offs=0;
+		llioGlobFlags=0;
+	}
+	virtual int llioGlobOne(char *filename,int flags=0) {
+		LOG_INFO(cEXBP,0,"llioGlobOne(%s)\n",filename);
+		int r=glob(filename,llioGlobFlags|flags,0,&llioGlobBuffer);
+		llioGlobFlags|=GLOB_APPEND;
+		return r;
+	}
+	virtual void llioGlob(void) {
+		for (int i=0;i<llioPossibleNameCount;i++) {
+			llioGlobOne(llioPossibleName[i]);
+		}
+	}
+	void llioGlobDone(void) {
+		globfree(&llioGlobBuffer);
+	}
+	virtual STATUS llioOpenOne(void);
+	STATUS llioMatchDevice(int *pOpened);
+	virtual STATUS llioMatchDevice(void);
+    public:
+	void resetTransport(ExTransport *pTransport);
+	char *llioNameGet(void) { return llioName; }
+    protected:
+	void llioNameSet(char *s) {
+		if (llioName) {
+			delete[] llioName;
+			llioName=0;
+		}
+		if (s) {
+			int len=strlen(s)+1;
+			llioName=new char[len];
+			if (!llioName) {
+				LOG_ERROR(cEXBP,0,cCauseNoMem,"llioNameSet: "
+					"'new char[%d]' failed!\n",len);
+			} else {
+				memcpy(llioName,s,len);
+			}
+		}
+	}
+	virtual char *llioGetDeviceID(int *pOpened)=0;
+	virtual STATUS llioPickInterfaces(void) {
+		llioInterface[EX_INTERFACE_MLC].active=(tryMlc || tryDot4);
+		llioInterface[EX_INTERFACE_PRINT].active=0;
+		return OK;
+	}
+	virtual STATUS llioSetup(ExInterface iface) { return OK; }
 	void llioClose(void);
 	virtual void llioReset(void) { }
+	STATUS llioDrainReverseData(void);
+	virtual int llioGetDrainBufferSize(int proposed) {
+		return proposed;
+	}
+	void llioStartPrintSubprocessWaitTimer(void) {
+		llioPrintSubprocessWaitTimer->start();
+	}
+	void llioCancelPrintSubprocessWaitTimer(void) {
+		llioPrintSubprocessWaitTimer->cancel();
+	}
+	int llioPrintInterfaceIsAvailable(void) {
+		if (!llioInterface[EX_INTERFACE_PRINT].active) return 0;
+		int r=pidHasExited(&llioInterface[EX_INTERFACE_PRINT].pid);
+		if (r>0) llioCancelPrintSubprocessWaitTimer();
+		return r;
+	}
+	void llioServicePrintSubprocessWaitTimer(void) {
+		if (llioPrintInterfaceIsAvailable()>0) {
+			// TODO: Deactivate-if-idle if desired.
+		}
+	}
 	void llioService(void);
-	void llioSchedulePoll(LlioPollEvent event);
-	virtual int llioReverseDataIsAvailable(void);
+	STATUS llioSubprocess(ExInterface iface,int fdSession=ERROR);
+	// virtual int llioShouldTryToReadExtra(ExInterface iface) {
+	//	return 0;
+	// }
+	virtual int _llioReverseDataIsAvailable(ExInterface iface) {
+		return 1;
+	}
+	int llioReverseDataIsAvailable(ExInterface iface) {
+		if (llioInterface[iface].fdDevice!=ERROR) {
+			return fdIsReadable(llioInterface[iface].fdDevice);
+		}
+		return _llioReverseDataIsAvailable(iface);
+	}
 	virtual STATUS llioForwardToReverse(void) { return OK; }
 	virtual STATUS llioReverseToForward(void) { return OK; }
-	virtual int llioRead(unsigned char *data,int datalen,int noblock);
-	virtual int llioWrite(unsigned char *data,int datalen);
+
+	/* iface!=EX_INTERFACE_NONE implies the following things:
+	 * - llioGenericRead: 0-byte internal reads don't mean EOF
+	 * - llioGenericRead: exact<0 means no retry of 0-byte read of 1st byte
+	 * - llioGenericRead: alarm() with retry
+	 * - llioGenericWrite: alarm() without retry */
+	virtual int _llioRead(ExInterface iface,
+	    unsigned char *data,int datalen,int exact) { return ERROR; }
+	int llioGenericRead(ExInterface iface,int fd,
+	    unsigned char *data,int datalen,int exact);
+	virtual int llioRead(ExInterface iface,
+	    unsigned char *data,int datalen,int exact) {
+		return llioGenericRead(iface,llioInterface[iface].fdDevice,
+			data,datalen,exact);
+	}
+	int fdRead(int fd,unsigned char *data,int datalen,int exact) {
+		return llioGenericRead(EX_INTERFACE_NONE,fd,
+			data,datalen,exact);
+	}
+
+	virtual int _llioWrite(ExInterface iface,
+	    const unsigned char *data,int datalen,int exact) { return ERROR; }
+	int llioGenericWrite(ExInterface iface,int fd,
+	    const unsigned char *data,int datalen,int exact);
+	virtual int llioWrite(ExInterface iface,
+	    const unsigned char *data,int datalen,int exact) {
+		return llioGenericWrite(iface,llioInterface[iface].fdDevice,
+			data,datalen,exact);
+	}
+	int fdWrite(int fd,const unsigned char *data,int datalen,int exact) {
+		return llioGenericWrite(EX_INTERFACE_NONE,fd,
+			data,datalen,exact);
+	}
+
     public:
 	void sendForwardData(ExBdr *pBdr,int priority) {
-		llioForwardBdrQueue->add(pBdr);
-		if (llioFd!=ERROR) fdRegister(llioFd,0,1);
+	    llioInterface[EX_INTERFACE_MLC].pForwardBdrQueue->add(pBdr);
+	    fdRegister(llioInterface[EX_INTERFACE_MLC].fdPipe.write,0,1);
 	}
-	void resetTransport(ExTransport *pTransport);
 };
 
 #endif
