@@ -1,6 +1,6 @@
 // ParPort.cpp
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,70 +20,33 @@
 
 /* Original author: David Paschal */
 
+#define ffs ffs__ParPort	// Workaround link failure on FreeBSD.
 #include "ParPort.h"
-#include <stdio.h>
 
-void ParPort::dump(void) {
-	printf("portType=%d\n",portType);
-	printf("debug=%d\n",debug);
-	dumpTimeval(TIMEVAL_SETUP_DELAY,"setupDelay");
-	dumpTimeval(TIMEVAL_STROBE_DELAY,"strobeDelay");
-	dumpTimeval(TIMEVAL_HOLD_DELAY,"holdDelay");
-	dumpTimeval(TIMEVAL_ECP_SETUP_DELAY,"ecpSetupDelay");
-	dumpTimeval(TIMEVAL_ECP_POST_HTR_DELAY,"ecpPostHtrDelay");
-	dumpTimeval(TIMEVAL_SIGNAL_TIMEOUT,"signalTimeout");
-	dumpTimeval(TIMEVAL_BUSY_TIMEOUT,"busyTimeout");
-	dumpTimeval(TIMEVAL_REVERSE_POLL_RATE,"reversePollRate");
-	printf("dead=%d\n",dead);
-	// statusWaitTimer
-	// ecpConfigA, ecpConfigB (see below)
-	printf("currentMode=0x%2.2X\n",currentMode);
-	printf("currentChannel=%d\n",currentChannel);
-	printf("htrCount=%d\n",htrCount);
-	printf("forwardMode=0x%2.2X\n",forwardMode);
-	printf("reverseMode=0x%2.2X\n",reverseMode);
-	printf("channel=%d\n",channel);
 
-	printf("status       =0x%2.2X\n",statusRead());
-	printf("control      =0x%2.2X\n",controlRead());
-	printf("ECP config A =0x%2.2X\n",ecpConfigA);
-	printf("ECP config B =0x%2.2X\n",ecpConfigB);
-	printf("ECP control  =0x%2.2X\n",ecpControlRead());
-}
+/*****************************************************************************\
+| Constructor, [destructor,] dump():
+\*****************************************************************************/
 
-void ParPort::dumpTimeval(TimevalIndex index,char *name) {
-	printf("%s(%d)={tv_sec=%ld,tv_usec=%ld}\n",
-		name,index,
-		timevalArray[index].tv_sec,
-		timevalArray[index].tv_usec);
-}
+ParPort::ParPort(int baseaddr0,int baseaddr1,int portType) {
+	int r;
 
-ParPort::ParPort(int portType) {
 	setPortType(portType,1);
-	setDebug(0);
-	setTimeval(TIMEVAL_SETUP_DELAY,0,100);
-	setTimeval(TIMEVAL_STROBE_DELAY,0,100);
-	setTimeval(TIMEVAL_HOLD_DELAY,0,100);
-	setTimeval(TIMEVAL_ECP_SETUP_DELAY,0,0);	/* TODO: Adjust? */
-	setTimeval(TIMEVAL_ECP_POST_HTR_DELAY,0,10000);
-	setTimeval(TIMEVAL_SIGNAL_TIMEOUT,0,100000);
-	setTimeval(TIMEVAL_BUSY_TIMEOUT,1,0);
-	setTimeval(TIMEVAL_REVERSE_POLL_RATE,0,10000);
+	initTimevals();
+	regInit(baseaddr0,baseaddr1);
+	ecpFifoSize=0;
 	currentMode=MODE_COMPAT;
 	currentChannel=0;
-	setHtrCount(4);
+	ecpChannelSetNeeded=0;
+	setHtrCount(DEFAULT_HTR_COUNT);
 	forwardMode=MODE_COMPAT;
 	reverseMode=MODE_NIBBLE;
-	channel=0;
-}
+	desiredChannel=0;
 
-void ParPort::reset(void) {
-	terminate();
-	dead=0;
-}
-
-void ParPort::postConstructor(void) {
-	int r;
+	if (getIoPortAccess()==ERROR) {
+		LOG_ERROR_FATAL(cEXBP,0,cCauseFuncFailed,
+			"Access denied to parallel port!\n");
+	}
 
 	/* Reset control lines in preparation for port type detection. */
 	controlWrite(CONTROL_NSTROBE|CONTROL_NAUTOFD|CONTROL_NINIT);
@@ -95,11 +58,13 @@ void ParPort::postConstructor(void) {
 		if (r==(ECP_CONTROL_MODE_BIDIRECTIONAL|
 		     ECP_CONTROL_ALWAYS_SET|ECP_CONTROL_EMPTY) &&
 		    r!=controlRead()) {
-			if (!portTypeIsEcpWithHwAssist()) goto ecpNoHwAssist;
 			ecpConfigRead();
+			if (!portTypeIsEcpWithHwAssist()) goto ecpNoHwAssist;
 			if ((ecpConfigA&0x70)!=0x10) {
-ecpNoHwAssist:
 				/* Set PORTTYPE_ECP if FIFO width>8 bits. */
+				LOG_WARN(cEXBP,0,"Using software ECP because "
+					"hardware word size exceeds 8 bits.\n");
+ecpNoHwAssist:
 				setPortType(PORTTYPE_ECP,1);
 			} else {
 				setPortType(PORTTYPE_ECPHW);
@@ -116,7 +81,9 @@ ecpNoHwAssist:
 
 	/* Try BPP. */
 	} else if (portTypeIsBidirectional()) {
-		controlSetClear(CONTROL_REVERSE_DATA,0);
+		controlSetClear(-1,
+			CONTROL_REVERSE_DATA,
+			0);
 
 		if (dataTest(43)==ERROR &&
 		    dataTest(34)==ERROR) {
@@ -125,169 +92,105 @@ ecpNoHwAssist:
 			setPortType(PORTTYPE_SPP,1);
 		}
 
-		controlSetClear(0,CONTROL_REVERSE_DATA);
+		controlSetClear(-1,
+			0,
+			CONTROL_REVERSE_DATA);
 		dataWrite(0);
 	}
 
-	reset();
-}
-
-void ParPort::preDestructor(void) {
-	reset();
-}
-
-ParPort::~ParPort() {
-}
-
-void ParPort::controlSetClear(int event,int set,int clear) {
-	if (debug) LOG_INFO(cEXBP,0,
-		"controlSetClear(event=%d,set=0x%2.2X,clear=0x%2.2X)\n",
-		event,set,clear);
-
-	controlSetClear(set,clear);
-}
-
-int ParPort::statusWaitSetClear(int event,TimevalIndex timeout,
-    int set,int clear) {
-	statusWaitTimer.setTimeout(lookupTimeval(timeout));
-
-	if (debug) LOG_INFO(cEXBP,0,
-		"statusWaitSetClear(event=%d,set=0x%2.2X,clear=0x%2.2X)\n",
-		event,set,clear);
-
-	while (!statusTestSetClear(set,clear)) {
-		if (event==43 && !statusReverseDataIsAvailable()) return ERROR;
-		if (statusWaitTimer.isTimedOut()) {
-			LOG_ERROR(cEXBP,0,cCausePeriphError,
-				"statusWaitSetClear(event=%d) timed out!\n",
-				event);
-			return ERROR;
+	if (portTypeIsEcpWithHwAssist() &&
+	    ecpControlSetMode(ECP_CONTROL_MODE_TEST)!=ERROR) {
+		while (!ecpFifoIsEmpty()) ecpDataRead();
+		while (!ecpFifoIsFull()) {
+			ecpDataWrite(0);
+			ecpFifoSize++;
+		}
+		while (!ecpFifoIsEmpty()) ecpDataRead();
+		ecpControlSetMode(ECP_CONTROL_MODE_BIDIRECTIONAL);
+		if (!ecpFifoSize) {
+			LOG_WARN(cEXBP,0,"Using software ECP because "
+				"hardware FIFO size is %d byte(s).\n",
+				ecpFifoSize);
+			setPortType(PORTTYPE_ECP,1);
 		}
 	}
 
-	return OK;
+	terminate();
 }
 
-int ParPort::ecpConfigRead(void) {
-	int r=ERROR;
+void ParPort::dump(void) {
+	printf("portType=%d\n",portType);
+	dumpTimeval(TIMEVAL_COMPAT_SETUP_DELAY,"compatSetupDelay");
+	dumpTimeval(TIMEVAL_COMPAT_STROBE_DELAY,"compatStrobeDelay");
+	dumpTimeval(TIMEVAL_COMPAT_HOLD_DELAY,"compatHoldDelay");
+	dumpTimeval(TIMEVAL_ECP_SETUP_DELAY,"ecpSetupDelay");
+	dumpTimeval(TIMEVAL_ECP_POST_HTR_DELAY,"ecpPostHtrDelay");
+	dumpTimeval(TIMEVAL_SIGNAL_TIMEOUT,"signalTimeout");
+	dumpTimeval(TIMEVAL_BUSY_TIMEOUT,"busyTimeout");
+	dumpTimeval(TIMEVAL_EPP_TERM_DELAY,"eppTermDelay");
+	printf("baseaddr=0x%3.3X\n",regaddr[REG_SPP_DATA]);
+	printf("basehigh=0x%3.3X\n",regaddr[REG_ECP_DATA]);
+	// iofd (FreeBSD only)
+	// statusWaitTimer
+	// ecpConfigA, ecpConfigB (see below)
+	printf("ecpFifoSize=%d\n",ecpFifoSize);
+	printf("currentMode=0x%2.2X\n",currentMode);
+	printf("currentChannel=%d\n",currentChannel);
+	printf("ecpChannelSetNeeded=%d\n",ecpChannelSetNeeded);
+	printf("htrCount=%d\n",htrCount);
+	printf("forwardMode=0x%2.2X\n",forwardMode);
+	printf("reverseMode=0x%2.2X\n",reverseMode);
+	printf("desiredChannel=%d\n",desiredChannel);
 
-	int old=ecpControlRead();
-	ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
-	if (ecpControlSetMode(ECP_CONTROL_MODE_CONFIGURATION)!=ERROR) {
-		ecpConfigA=ecpDataRead();
-		ecpConfigB=ecpStatusRead();
-		ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
-		r=OK;
-	}
-	ecpControlSetMode(old);
-
-	return r;
+	printf("status       =0x%2.2X\n",statusRead());
+	printf("control      =0x%2.2X\n",controlRead());
+    if (portTypeIsEcp()) {
+	printf("ECP config A =0x%2.2X\n",ecpConfigA);
+	printf("ECP config B =0x%2.2X\n",ecpConfigB);
+	printf("ECP control  =0x%2.2X\n",ecpControlRead());
+    }
 }
 
-int ParPort::ecpControlGetMode(void) {
-	if (portTypeIsEcp()) {
-		return ecpControlRead()&ECP_CONTROL_MODE_MASK;
-	}
-	if (portTypeIsBidirectional()) {
-		return ECP_CONTROL_MODE_BIDIRECTIONAL;
-	}
-	return ECP_CONTROL_MODE_UNIDIRECTIONAL;
-}
 
-int ParPort::ecpControlSetMode(int mode) {
-	if (portTypeIsEcp()) {
-		ecpControlWrite(mode);
-		return OK;
-	}
-	if (portTypeIsBidirectional()) {
-		if (mode>ECP_CONTROL_MODE_BIDIRECTIONAL) return ERROR;
-		return OK;
-	}
-	if (mode>ECP_CONTROL_MODE_UNIDIRECTIONAL) return ERROR;
-	return OK;
-}
-
-int ParPort::ecpFifoWaitNotFull(TimevalIndex timeout) {
-	if (!ecpHwAssistIsEnabled()) return OK;
-
-	statusWaitTimer.setTimeout(lookupTimeval(timeout));
-
-	while (ecpFifoIsFull()) {
-		if (statusWaitTimer.isTimedOut()) {
-			/* TODO: Host transfer recovery if ECP/forward. */
-
-			LOG_ERROR(cEXBP,0,cCausePeriphError,
-				"ecpFifoWaitNotFull timed out!\n");
-			return ERROR;
-		}
-	}
-
-	return OK;
-}
-
-int ParPort::ecpFifoWaitNotEmpty(TimevalIndex timeout) {
-	if (!ecpHwAssistIsEnabled()) return OK;
-
-	statusWaitTimer.setTimeout(lookupTimeval(timeout));
-
-	while (ecpFifoIsEmpty()) {
-		if (statusWaitTimer.isTimedOut()) {
-			LOG_ERROR(cEXBP,0,cCausePeriphError,
-				"ecpFifoWaitNotEmpty timed out!\n");
-			return ERROR;
-		}
-	}
-
-	return OK;
-}
-
-int ParPort::ecpFifoWaitEmpty(TimevalIndex timeout) {
-	if (!ecpHwAssistIsEnabled()) return OK;
-
-	statusWaitTimer.setTimeout(lookupTimeval(timeout));
-
-	while (!ecpFifoIsEmpty()) {
-		if (statusWaitTimer.isTimedOut()) {
-			/* TODO: Host transfer recovery if ECP/forward. */
-
-			LOG_ERROR(cEXBP,0,cCausePeriphError,
-				"ecpFifoWaitEmpty timed out!\n");
-			return ERROR;
-		}
-	}
-
-	return OK;
-}
+/*****************************************************************************\
+| Low-level 1284 mode/direction negotiation:
+\*****************************************************************************/
 
 int ParPort::negotiate(int mode) {
 	int selectSet=0,selectClear=0,r;
 
-	if (!isCompat()) {
+	if (!modeIsCompat()) {
 		LOG_ERROR(cEXBP,0,cCauseBadState,
 			"negotiate(mode=0x%2.2X): not compatibility mode!\n",
 			mode);
 		return ERROR;
 	}
 
-	ecpFifoWaitEmpty(TIMEVAL_BUSY_TIMEOUT);
+	ecpFifoWaitForEmpty(TIMEVAL_BUSY_TIMEOUT);
 
 	/* Disable hardware-assisted compatibility mode. */
-	r=ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
-	if (r==ERROR) {
-		LOG_ERROR(cEXBP,0,cCauseFuncFailed,
-			"negotiate(mode=0x%2.2X): "
-			"not supported by hardware!\n",
-			mode);
-		return ERROR;
+	ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
+
+	/* EPP requires bidirectional capability. */
+	if (modeIsEpp(mode)) {
+		r=ecpControlSetMode(ECP_CONTROL_MODE_BIDIRECTIONAL);
+		if (r==ERROR) {
+			LOG_ERROR(cEXBP,0,cCauseFuncFailed,
+				"negotiate(mode=0x%2.2X): "
+				"not supported by hardware!\n",
+				mode);
+			return ERROR;
+		}
 	}
 
 	/* Event 0: Write extensibility request to data lines. */
 	dataWrite(mode);
 
-	/* Event 1: nSelectIn=1, nAutoFd=0, nStrobe=1, nInit=1. */
+	/* Event 1: nSelectIn=1, nAutoFd=0, nStrobe=1, nInit=1,
+	 * drive data lines. */
 	controlSetClear(1,
 		CONTROL_NSELECTIN|CONTROL_NSTROBE|CONTROL_NINIT,
-		CONTROL_NAUTOFD);
+		CONTROL_NAUTOFD|CONTROL_REVERSE_DATA);
 
 	/* Event 2: PError=1, Select=1, nFault=1, nAck=0. */
 	r=statusWaitSetClear(2,TIMEVAL_SIGNAL_TIMEOUT,
@@ -299,7 +202,7 @@ int ParPort::negotiate(int mode) {
 	controlSetClear(3,
 		0,
 		CONTROL_NSTROBE);
-	PolledTimer::delay(lookupTimeval(TIMEVAL_STROBE_DELAY));
+	PolledTimer::delay(lookupTimeval(TIMEVAL_COMPAT_STROBE_DELAY));
 
 	/* Event 4: nStrobe=1, nAutoFd=1. */
 	controlSetClear(4,
@@ -313,21 +216,22 @@ int ParPort::negotiate(int mode) {
 	if (r==ERROR) goto abort;
 
 	/* Event 5: Select=0 for nibble-0, =1 for other modes. */
-	if (mode) {
-		selectSet=STATUS_SELECT;
-	} else {
+	if (modeIsNibble0(mode)) {
 		selectClear=STATUS_SELECT;
+	} else {
+		selectSet=STATUS_SELECT;
 	}
-	currentMode=!mode;
 	if (!statusTestSetClear(selectSet,selectClear)) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"negotiate(mode=0x%2.2X): rejected!\n",mode);
+		LOG_WARN(cEXBP,0,
+			"negotiate(mode=0x%2.2X): rejected by peripheral!\n",
+			mode);
+		currentMode=!mode;
 		goto abort;
 	}
-	currentMode=mode;
+	currentMode=mode&MODE_1284_MASK;
 
 	/* Extra signalling for ECP mode. */
-	if (isEcp()) {
+	if (modeIsEcp()) {
 		/* Event 30: nAutoFd=0. */
 		controlSetClear(30,
 			0,
@@ -339,12 +243,25 @@ int ParPort::negotiate(int mode) {
 			0);
 		if (r==ERROR) goto abort;
 
+		/* Event 33: Host is idle (nAutoFd=1). */
+		controlSetClear(33,
+			CONTROL_NAUTOFD,
+			0);
+
 		/* Enable hardware-assisted ECP mode if possible. */
-		if (mode&MODE_HW_ASSIST) {
+		if (modeIsHwAssist(mode)) {
 			ecpControlSetMode(ECP_CONTROL_MODE_ECP);
+			currentMode|=MODE_HW_ASSIST;
 		}
 
 		currentChannel=0;
+
+	/* Extra signalling for EPP mode. */
+	} else if (modeIsEpp()) {
+		/* Event 14: Tristate data lines. */
+		controlSetClear(14,
+			CONTROL_REVERSE_DATA,
+			0);
 	}
 
 	return OK;
@@ -356,64 +273,88 @@ abort:
 
 int ParPort::terminate(int mode) {
 	int r,selectSet=0,selectClear=0;
+	int priorStatus=statusRead();
 
-	if (isEcp()) {
+    if (modeIsEpp()) {
+	/* Event 68: nInit=0. */
+	controlSetClear(68,
+		CONTROL_NSELECTIN|CONTROL_NAUTOFD|CONTROL_NSTROBE,
+		CONTROL_NINIT);
+	PolledTimer::delay(lookupTimeval(TIMEVAL_EPP_TERM_DELAY));
+
+	/* Event 69: nInit=1, nSelectIn=0, drive data lines. */
+	controlSetClear(69,
+		CONTROL_NINIT,
+		CONTROL_NSELECTIN|CONTROL_REVERSE_DATA);
+
+	ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
+
+    } else {
+	if (modeIsEcp()) {
 		ecpRevToFwd();
 	}
 
-	ecpFifoWaitEmpty(TIMEVAL_BUSY_TIMEOUT);
+	ecpFifoWaitForEmpty(TIMEVAL_BUSY_TIMEOUT);
 
 	/* Disable hardware-assisted ECP mode if necessary. */
 	ecpControlSetMode(ECP_CONTROL_MODE_UNIDIRECTIONAL);
 
-	/* Event 22: nSelectIn=0, nAutoFd=1, nStrobe=1, nInit=1. */
+	/* Event 22: nSelectIn=0, nAutoFd=1, nStrobe=1, nInit=1,
+	 * drive data lines. */
 	controlSetClear(22,
 		CONTROL_NAUTOFD|CONTROL_NSTROBE|CONTROL_NINIT,
-		CONTROL_NSELECTIN);
+		CONTROL_NSELECTIN|CONTROL_REVERSE_DATA);
 
-	if (isCompat()) goto done;
+	if (!modeIsCompat()) {
+		/* Event 23: Busy=1, nFault=1. */
+		/* Event 24: nAck=0, Select=toggled
+		 * (1 if terminating nibble-0, 0 otherwise). */
+		if (currentMode) {
+			selectClear=STATUS_SELECT;
+		} else {
+			selectSet=STATUS_SELECT;
+		}
+		r=statusWaitSetClear(23,TIMEVAL_SIGNAL_TIMEOUT,
+			STATUS_BUSY|STATUS_NFAULT|selectSet,
+			STATUS_NACK|selectClear);
 
-	/* Event 23: Busy=1, nFault=1. */
-	/* Event 24: nAck=0, Select=toggled
-	 * (1 if terminating nibble-0, 0 otherwise). */
-	if (currentMode) {
-		selectClear=STATUS_SELECT;
-	} else {
-		selectSet=STATUS_SELECT;
+		if (r!=ERROR) {
+			/* Event 25: nAutoFd=0. */
+			controlSetClear(25,
+				0,
+				CONTROL_NAUTOFD);
+		}
+
+		/* Event 26: PError, nFault, Select to compat. mode values. */
+		/* Event 27: nAck=1. */
+		r=statusWaitSetClear(27,TIMEVAL_SIGNAL_TIMEOUT,
+			STATUS_NACK,
+			0);
+		if (r==ERROR) goto abort;
+
+		/* Event 28: nAutoFd=1. */
+		controlSetClear(28,
+			CONTROL_NAUTOFD,
+			0);
+
+		/* Event 29: Busy to compatibility mode value. */
 	}
+    }
+
 	currentMode=MODE_COMPAT;
-	r=statusWaitSetClear(23,TIMEVAL_SIGNAL_TIMEOUT,
-		STATUS_BUSY|STATUS_NFAULT|selectSet,
-		STATUS_NACK|selectClear);
-	if (r==ERROR) goto abort;
 
-	/* Event 25: nAutoFd=0. */
-	controlSetClear(25,
-		0,
-		CONTROL_NAUTOFD);
-
-	/* Event 26: PError, nFault, Select to compat. mode values. */
-	/* Event 27: nAck=1. */
-	r=statusWaitSetClear(27,TIMEVAL_SIGNAL_TIMEOUT,
-		STATUS_NACK,
-		0);
-	if (r==ERROR) goto abort;
-
-	/* Event 28: nAutoFd=1. */
-	controlSetClear(28,
-		CONTROL_NAUTOFD,
-		0);
-
-	/* Event 29: Busy to compatibility mode value. */
-done:
 	/* Enable hardware-assisted compatibility mode if possible. */
-	if (mode&MODE_HW_ASSIST) {
+	if (modeIsHwAssist(mode)) {
 		ecpControlSetMode(ECP_CONTROL_MODE_FAST_CENTRONICS);
+		currentMode|=MODE_HW_ASSIST;
 	}
 
 	return OK;
 
 abort:
+	LOG_WARN(cEXBP,0,
+		"terminate: priorStatus=0x%2.2X.\n",priorStatus);
+	currentMode=MODE_COMPAT;
 	terminate();	/* This is recursive.  OK if MODE_COMPAT. */
 	return ERROR;
 }
@@ -421,10 +362,10 @@ abort:
 int ParPort::ecpFwdToRev(int mode) {
 	int r;
 
-	if (isEcpReverse()) return OK;
-	if (!isEcpForward()) return ERROR;
+	if (modeIsEcpReverse()) return OK;
+	if (!modeIsEcpForward()) return ERROR;
 
-	ecpFifoWaitEmpty(TIMEVAL_BUSY_TIMEOUT);
+	ecpFifoWaitForEmpty(TIMEVAL_BUSY_TIMEOUT);
 
 	/* Disable hardware-assisted ECP mode if necessary. */
 	r=ecpControlSetMode(ECP_CONTROL_MODE_BIDIRECTIONAL);
@@ -434,15 +375,12 @@ int ParPort::ecpFwdToRev(int mode) {
 		return ERROR;
 	}
 
-	/* Event 38: data lines tri-stated, nAutoFd=0. */
+	/* Event 38: nAutoFd=0, tristate data lines. */
+	/* There really should be a proper delay between these two events. */
+	/* Event 39: nInit=0. */
 	controlSetClear(38,
 		CONTROL_REVERSE_DATA,
-		CONTROL_NAUTOFD);
-
-	/* Event 39: nInit=0. */
-	controlSetClear(39,
-		0,
-		CONTROL_NINIT);
+		CONTROL_NAUTOFD|CONTROL_NINIT);
 
 	/* Event 40: PError=0. */
 	r=statusWaitSetClear(40,TIMEVAL_SIGNAL_TIMEOUT,
@@ -450,33 +388,43 @@ int ParPort::ecpFwdToRev(int mode) {
 		STATUS_PERROR);
 	if (r==ERROR) return ERROR;
 
-	currentMode|=MODE_REVERSE;
-
-	/* Enable hardware-assisted ECP mode if possible. */
-	if (mode&MODE_HW_ASSIST) {
-		ecpControlSetMode(ECP_CONTROL_MODE_ECP);
-	}
+	currentMode=(currentMode&~MODE_HW_ASSIST)|MODE_REVERSE|
+		(mode&MODE_HW_ASSIST);
 
+	/* We enable hardware-assisted ECP mode later if desired. */
 	return OK;
 }
 
 int ParPort::ecpRevToFwd(int mode) {
 	int r;
 
-	if (isEcpForward()) return OK;
-	if (!isEcpReverse()) return ERROR;
+	if (modeIsEcpForward()) return OK;
+	if (!modeIsEcpReverse()) return ERROR;
 
 	/* For bounded ECP, wait for nFault=1.  Ignore timeout. */
-	if (isBoundedEcp()) {
+	if (modeIsBoundedEcp()) {
 		statusWaitSetClear(147,TIMEVAL_BUSY_TIMEOUT,
 			STATUS_NFAULT,
 			0);
 	}
 
-	/* Event 47: nInit=1. */
+	/* Event 47: nInit=1.
+	 * Also force nAutoFd=0 in case we're using HW assistance. */
 	controlSetClear(47,
 		CONTROL_NINIT,
-		0);
+		CONTROL_NAUTOFD);
+
+	/* Print error message if FIFO has data. */
+	if (ecpHwAssistIsEnabled()) {
+		while (!ecpFifoIsEmpty()) {
+			LOG_ERROR(cEXBP,0,cCausePeriphError,
+				"ecpRevToFwd: lost reverse data 0x%2.2X!\n",
+				ecpDataRead());
+		}
+	}
+
+	/* Disable hardware-assisted ECP mode if necessary. */
+	ecpControlSetMode(ECP_CONTROL_MODE_BIDIRECTIONAL);
 
 	/* Event 48: nAck=1, Busy/nFault=valid. */
 	/* Event 49: PError=1. */
@@ -485,60 +433,51 @@ int ParPort::ecpRevToFwd(int mode) {
 		0);
 	if (r==ERROR) return ERROR;
 
-	/* Print error message if FIFO has data. */
-	while (!ecpFifoIsEmpty()) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"ecpRevToFwd: lost reverse data 0x%2.2X!\n",
-			ecpDataRead());
-	}
-
-	/* Disable hardware-assisted ECP mode if necessary. */
-	ecpControlSetMode(ECP_CONTROL_MODE_BIDIRECTIONAL);
-
-	/* Event 33: Host resumes driving data lines. */
+	/* Event 33: Host is idle (nAutoFd=1), drive data lines. */
 	controlSetClear(33,
-		0,
+		CONTROL_NAUTOFD,
 		CONTROL_REVERSE_DATA);
 
-	currentMode&=~MODE_REVERSE;
+	currentMode&=~(MODE_REVERSE|MODE_HW_ASSIST);
 
 	/* Enable hardware-assisted ECP mode if possible. */
-	if (mode&MODE_HW_ASSIST) {
+	if (modeIsHwAssist(mode)) {
 		ecpControlSetMode(ECP_CONTROL_MODE_ECP);
+		currentMode|=MODE_HW_ASSIST;
 	}
 
 	return OK;
 }
 
-int ParPort::writeCompatByte(int c) {
-	/* TODO: Implement! */
 
-	currentChannel=0;
+/*****************************************************************************\
+| Forward data transfers:
+\*****************************************************************************/
 
-	PolledTimer::delay(lookupTimeval(TIMEVAL_SETUP_DELAY));
+int ParPort::writeEcp(const unsigned char *buffer,int len,
+    int isCommand) {
+	int r,htrCountdown=htrCount,countup=0;
 
-	PolledTimer::delay(lookupTimeval(TIMEVAL_STROBE_DELAY));
+	/* Event 34: Write command line (data lines written below). */
+	if (!isCommand) {
+		controlSetClear(34,
+			CONTROL_NAUTOFD,
+			0);
+	} else {
+		controlSetClear(34,
+			0,
+			CONTROL_NAUTOFD);
+	}
 
-	PolledTimer::delay(lookupTimeval(TIMEVAL_HOLD_DELAY));
-
-	return ERROR;
-}
-
-int ParPort::writeEcpByte(int c) {
-	int r,htrCountdown=htrCount,cmd=c&ECP_COMMAND;
-
-retry:
+    while (42) {
 	/* Event 32: Busy=0. */
 	r=statusWaitSetClear(32,TIMEVAL_BUSY_TIMEOUT,
 		0,
 		STATUS_BUSY);
-	if (r==ERROR) goto abort;
+	if (len<=0 || r==ERROR) break;
 
-	/* Event 34: Write data and command lines. */
-	dataWrite(c);
-	controlSetClear(34,
-		cmd?0:CONTROL_NAUTOFD,
-		cmd?CONTROL_NAUTOFD:0);
+	/* Event 34: Write data lines (command line written above). */
+	dataWrite(*buffer);
 	PolledTimer::delay(lookupTimeval(TIMEVAL_ECP_SETUP_DELAY));
 
 	/* Event 35: nStrobe=0. */
@@ -547,10 +486,11 @@ retry:
 		CONTROL_NSTROBE);
 
 	/* Event 36: Busy=1. */
-	r=statusWaitSetClear(36,TIMEVAL_SIGNAL_TIMEOUT,STATUS_BUSY,0);
+	r=statusWaitSetClear(36,TIMEVAL_SIGNAL_TIMEOUT,
+		STATUS_BUSY,
+		0);
 	if (r==ERROR) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"Host transfer recovery!\n");
+		LOG_WARN(cEXBP,0,"Host transfer recovery!\n");
 
 		/* Event 72: nInit=0. */
 		controlSetClear(72,
@@ -567,12 +507,12 @@ retry:
 			CONTROL_NINIT|CONTROL_NSTROBE,
 			0);
 
-		if (htrCountdown<=1) return ERROR;
+		if (r==ERROR || htrCountdown<=1) break;
 
 		PolledTimer::delay(lookupTimeval(TIMEVAL_ECP_POST_HTR_DELAY));
 
 		htrCountdown--;
-		goto retry;
+		continue;
 	}
 
 	/* Event 37: nStrobe=1. */
@@ -580,82 +520,81 @@ retry:
 		CONTROL_NSTROBE,
 		0);
 
-	/* Event 32 (again): Busy=0. */
-	r=statusWaitSetClear(132,TIMEVAL_BUSY_TIMEOUT,
-		0,
-		STATUS_BUSY);
-	/* if (r==ERROR) goto abort; */
-
-	return OK;
-abort:
-	return ERROR;
-}
-
-int ParPort::writeHwByte(int c) {
-	/* TODO: Figure out why this doesn't work. */
-
-	if (ecpFifoWaitNotFull(TIMEVAL_BUSY_TIMEOUT)==ERROR) return ERROR;
-
-	if (c&ECP_COMMAND) {
-		dataWrite(c&~ECP_COMMAND);
-	} else {
-		ecpDataWrite(c);
-	}
-
-	return OK;
-}
-
-int ParPort::writeByte(int c) {
-	if (isEcpForward()) {
-		if (!ecpHwAssistIsEnabled()) {
-			return writeEcpByte(c);
-		}
-		return writeHwByte(c);
-	}
-
-	if (isCompat()) {
-		if (!ecpHwAssistIsEnabled()) {
-			return writeCompatByte(c);
-		}
-		return writeHwByte(c);
-	}
-
-	LOG_ERROR(cEXBP,0,cCauseBadState,
-		"writeByte: invalid mode=0x%4.4X!\n",currentMode);
-	return ERROR;
-}
-
-int ParPort::writeByte(int c,int count) {
-	int i;
-
-	for (i=0;i<count;i++) {
-		if (writeByte(c)==ERROR) break;
-	}
-
-	return i;
-}
-
-int ParPort::write(unsigned char *buffer,int len) {
-	int countup=0;
-
-	while (countup<len) {
-		if (writeByte(buffer[countup++])==ERROR) break;
-	}
+	buffer++;
+	countup++;
+	len--;
+    }
 
 	return countup;
 }
 
-int ParPort::readNibble(int index) {
-	int r,nibble;
+int ParPort::writeEpp(const unsigned char *buffer,int len,
+    int isAddress) {
+	int countup=0,eppSet,eppClear,r;
+	eppGetSetClearLines(isAddress,&eppSet,&eppClear);
 
+	/* Event 56/62 (partial): nStrobe=0(=write), drive data lines. */
+	controlSetClear(56,
+		0,
+		CONTROL_NSTROBE|CONTROL_REVERSE_DATA);
+
+    while (42) {
+	/* Event 60: Busy=0. */
+	r=statusWaitSetClear(60,TIMEVAL_BUSY_TIMEOUT,
+		0,
+		STATUS_BUSY);
+	if (len<=0 || r==ERROR) break;
+
+	/* Event 56/62 (partial): write data lines,
+	 * isAddress?nSelectIn=0:nAutoFd=0. */
+	dataWrite(*buffer);
+	controlSetClear(62,
+		eppSet,
+		eppClear);
+
+	/* Event 58: Busy=1. */
+	r=statusWaitSetClear(58,TIMEVAL_BUSY_TIMEOUT,
+		STATUS_BUSY,
+		0);
+	/* We check the return code below. */
+
+	/* Event 59/63: nAutoFd=1, nSelectIn=1. */
+	controlSetClear(59,
+		CONTROL_NAUTOFD|CONTROL_NSELECTIN,
+		0);
+
+	if (r==ERROR) break;
+
+	buffer++;
+	countup++;
+	len--;
+    }
+
+	/* Event 61: nStrobe=1, tristate data lines. */
+	controlSetClear(61,
+		CONTROL_NSTROBE|CONTROL_REVERSE_DATA,
+		0);
+
+	return countup;
+}
+
+
+/*****************************************************************************\
+| Reverse data transfers:
+\*****************************************************************************/
+
+int ParPort::readNibble(unsigned char *buffer,int len) {
+	int r,countup=0,index=0;
+
+    while (len>0) {
 	/* For the first nibble make sure the peripheral is asserting
 	 * nDataAvail (nFault). */
-	if (!index && !statusTestSetClear(0,STATUS_NFAULT)) {
-		LOG_WARN(cEXBP,0,"readNibble: index=0, nFault=1!\n");
-		return ERROR_NO_DATA;
+	if (!index && !statusReverseDataIsAvailable()) {
+		LOG_WARN(cEXBP,0,"readNibble: no more data!\n");
+		break;
 	}
 
-	/* Event 7 or 12: nAutoFd=0. */
+	/* Event 7/12: nAutoFd=0. */
 	controlSetClear(7,
 		0,
 		CONTROL_NAUTOFD);
@@ -665,12 +604,19 @@ int ParPort::readNibble(int index) {
 		0,
 		STATUS_NACK);
 	if (r==ERROR) {
-		return ERROR_NO_DATA;
+		/* Undo event 7/12: nAutoFd=1. */
+		controlSetClear(-7,
+			CONTROL_NAUTOFD,
+			0);
+		break;
 	}
 
 	/* Event 8: Read nibble. */
-	nibble=statusReadNibble();
-	if (index) nibble<<=4;
+	if (!index) {
+		*buffer=statusReadLowNibble();
+	} else {
+		*buffer|=statusReadHighNibble();
+	}
 
 	/* Event 10: nAutoFd=1. */
 	controlSetClear(10,
@@ -681,31 +627,33 @@ int ParPort::readNibble(int index) {
 	r=statusWaitSetClear(11,TIMEVAL_SIGNAL_TIMEOUT,
 		STATUS_NACK,
 		0);
-	if (r==ERROR) goto abort;
+	/* We check the return code below. */
 
-	return nibble;
-abort:
-	return ERROR;
+	index^=1;
+	if (!index) {
+		buffer++;
+		countup++;
+		len--;
+	}
+
+	if (r==ERROR) break;
+    }
+
+	return countup;
 }
 
-int ParPort::readEcpByte(void) {
-	int r,c;
+int ParPort::readEcpSw(unsigned char *buffer,int len) {
+	int r,countup=0;
 
+    while (len>0) {
 	/* Event 43: nAck=0. */
 	r=statusWaitSetClear(43,TIMEVAL_BUSY_TIMEOUT,
 		0,
 		STATUS_NACK);
-	if (r==ERROR) {
-		return ERROR_NO_DATA;
-	}
+	if (r==ERROR) break;
 
-	/* Event 42: Peripheral drives data lines, Busy=nCmd. */
-	c=dataRead();
-	if (statusTestSetClear(0,STATUS_BUSY)) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"readEcpByte: received ECP command=0x%2.2X!\n",c);
-		c|=ECP_COMMAND;
-	}
+	/* Event 42: Peripheral drives data lines, Busy=nCmd(ignored). */
+	*buffer=dataRead();
 
 	/* Event 44: nAutoFd=1. */
 	controlSetClear(44,
@@ -723,114 +671,102 @@ int ParPort::readEcpByte(void) {
 		0,
 		CONTROL_NAUTOFD);
 
+	buffer++;
+	countup++;
+	len--;
+
 	/* Event 41: Peripheral is idle. */
+	if (r==ERROR) break;
+    }
 
-	if (r==ERROR) return ERROR;
-	return c;
-}
-
-int ParPort::readHwByte(void) {
-	/* TODO: Figure out why this doesn't work. */
-
-	if (ecpFifoWaitNotEmpty(TIMEVAL_BUSY_TIMEOUT)==ERROR) return ERROR_NO_DATA;
-
-	return ecpDataRead();
-}
-
-int ParPort::readByte(void) {
-	int n0,n1;
-
-	if (isNibble()) {
-		n0=readNibble(0);
-		if (n0<0) return ERROR;
-		n1=readNibble(1);
-		if (n1<0) {
-			LOG_ERROR(cEXBP,0,cCausePeriphError,
-				"readByte: error reading second nibble!\n");
-			return ERROR;
-		}
-		return (n0|n1);
-	}
-
-	if (isEcpReverse()) {
-		if (!ecpHwAssistIsEnabled()) {
-			return readEcpByte();
-		}
-		return readHwByte();
-	}
-
-	LOG_ERROR(cEXBP,0,cCauseBadState,
-		"readByte: invalid mode=0x%4.4X!\n",currentMode);
-	return ERROR;
-}
-
-int ParPort::read(unsigned char *buffer,int len) {
-	int c,countup=0;
-
-	while (countup<len) {
-		c=readByte();
-		if (c==ERROR) break;
-		buffer[countup++]=c;
-	}
-
-	if (isDead()) return ERROR;
 	return countup;
 }
 
-int ParPort::flushReverseData(void) {
-	int i=0;
-	while (readByte()!=ERROR) i++;
-	return i;
+int ParPort::readEpp(unsigned char *buffer,int len,int isAddress) {
+	int countup=0,eppSet,eppClear,r;
+	eppGetSetClearLines(isAddress,&eppSet,&eppClear);
+
+	/* Event 64/67 (partial): nStrobe=1(=read), tristate data lines. */
+	controlSetClear(64,
+		CONTROL_NSTROBE|CONTROL_REVERSE_DATA,
+		0);
+
+    while (42) {
+	/* Event 60: Busy=0. */
+	r=statusWaitSetClear(60,TIMEVAL_BUSY_TIMEOUT,
+		0,
+		STATUS_BUSY);
+	if (len<=0 || r==ERROR) break;
+
+	/* This is non-standard for EPP: */
+	if (!statusReverseDataIsAvailable()) break;
+
+	/* Event 64/67 (partial): isAddress?nSelectIn=0:nAutoFd=0. */
+	controlSetClear(67,
+		eppSet,
+		eppClear);
+
+	/* Event 58: Busy=1. */
+	int r=statusWaitSetClear(158,TIMEVAL_BUSY_TIMEOUT,
+		STATUS_BUSY,
+		0);
+	/* We check the return code below. */
+
+	/* Event 65: Read data lines. */
+	*buffer=dataRead();
+
+	/* Event 59/63: nAutoFd=1, nSelectIn=1. */
+	controlSetClear(59,
+		CONTROL_NAUTOFD|CONTROL_NSELECTIN,
+		0);
+
+	if (r==ERROR) break;
+
+	buffer++;
+	countup++;
+	len--;
+    }
+
+	return countup;
 }
 
-int ParPort::readDeviceIDLength(void) {
-	int c,length;
-
-	c=readByte();
-	if (c==ERROR) {
-		return ERROR;
-	}
-	length=c<<8;
-	c=readByte();
-	if (c==ERROR) {
-		return ERROR;
-	}
-	length|=c;
-	length-=2;
-	if (length<0) length=ERROR;
-
-	return length;
-}
-
-unsigned char *ParPort::getDeviceID(void) {
-	unsigned char *buffer=0;
+char *ParPort::getDeviceID(void) {
+	unsigned char slength[2];
+	char *buffer=0;
 	int r,length;
 
 	r=negotiate(MODE_NIBBLE|MODE_DEVICE_ID);
 	if (r==ERROR) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
+		LOG_WARN(cEXBP,0,
 			"getDeviceID: negotiate() returns %d!\n",r);
 		goto abort;
 	}
 
-	length=readDeviceIDLength();
-	if (length==ERROR) {
+	r=read(slength,2);
+	if (r!=2) {
 		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"getDeviceID: readDeviceIDLength() returns %d!\n",r);
+			"getDeviceID: read %d bytes, expected=2!\n",r);
+		goto abort;
+	}
+	length=((slength[0]<<8)|slength[1])-2;
+	if (length<0) {
+		LOG_ERROR(cEXBP,0,cCausePeriphError,
+			"getDeviceID: invalid length=%d!\n",length);
 		goto abort;
 	}
 
-	buffer=new unsigned char[length+1];
+	buffer=new char[length+1];
 	if (!buffer) {
 		LOG_ERROR(cEXBP,0,cCauseNoMem,
-			"getDeviceID: error allocating buffer!\n");
+			"getDeviceID: error allocating %d-byte buffer!\n",
+			length+1);
 		goto abort;
 	}
 
-	r=read(buffer,length);
+	r=read((unsigned char *)buffer,length);
 	if (r!=length) {
 		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"getDeviceID: read %d bytes, expected %d!\n",
+			"getDeviceID: read %d bytes, expected=%d!\n",
 			r,length);
 		if (r<0) goto abort;
 	}
@@ -851,49 +787,3 @@ abort:
 	}
 	goto done;
 }
-
-int ParPort::setModes(int forwardMode,int reverseMode) {
-	SETTHIS(forwardMode);
-	SETTHIS(reverseMode);
-
-	/* TODO: Compare current and new forward mode to decide what to do. */
-
-	if (negotiate(forwardMode)==ERROR) {
-		LOG_ERROR(cEXBP,0,cCausePeriphError,
-			"setModes: negotiate(0x%2.2X) failed!\n",forwardMode);
-		return ERROR;
-	}
-
-	return OK;
-}
-
-int ParPort::startReverse(void) {
-	// if (!statusTestSetClear(0,STATUS_NFAULT)) return ERROR;
-
-	if (isEcp(reverseMode)) {
-		return ecpFwdToRev(reverseMode);
-	}
-
-	/* TODO: Make this more general! */
-	if (terminate()==ERROR ||
-	    negotiate(reverseMode)==ERROR) {
-		return ERROR;
-	}
-
-	return OK;
-}
-
-int ParPort::finishReverse(void) {
-	if (isEcp(reverseMode)) {
-		return ecpRevToFwd(forwardMode);
-	}
-
-	/* TODO: Make this more general! */
-	if (terminate()==ERROR ||
-	    negotiate(forwardMode)==ERROR ||
-	    writeEcpChannel(channel)==ERROR) {
-		return ERROR;
-	}
-
-	return OK;
-}
