@@ -1,6 +1,6 @@
 // ExMlcCommandChannel.cpp
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -96,7 +96,7 @@ const int ExMlcCommandChannel::DOT4_LEN_ERROR;
 
 ExMlcCommandChannel::ExMlcCommandChannel(ExMlcTransport *pMlcTransport,
     int channel,ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,int channelCount,
-    int final=1):
+    int final):
       ExMlcTransportChannel(pMlcTransport,channel,COMMAND_SOCKET,
        pMgr,pPhysicalPort,this,0) {
 	disableCreditCommands=1;
@@ -163,6 +163,7 @@ void ExMlcCommandChannel::reset(int rtype) {
 		allowErrorPackets=1;
 		tknobGetWorkingValue(port,EX_KNOB_MLC_SEND_ERROR_PACKETS,
 			&allowErrorPackets);
+		lastRequest=ERROR;
 		lastPsid=ERROR;
 		lastSsid=ERROR;
 
@@ -197,6 +198,10 @@ void ExMlcCommandChannel::registerStuff(char *title) {
 		&allowErrorPackets,
 		"%s[%d,%d].allowErrorPackets",
 		title,port,channel);
+	nettestRegister(NETTEST_TYPE_INT32,
+		&lastRequest,
+		"%s[%d,%d].lastRequest",
+		title,port,channel);
 	nettestRegister(NETTEST_TYPE_INT32,
 		&lastPsid,
 		"%s[%d,%d].lastPsid",
@@ -227,6 +232,8 @@ void ExMlcCommandChannel::dump(void) {
 		(int)pCommandReplyTimeoutMsg);
 	printf("allowErrorPackets=%d\n",
 		allowErrorPackets);
+	printf("lastRequest=0x%2.2X\n",
+		lastRequest);
 	printf("lastPsid=%d\n",
 		lastPsid);
 	printf("lastSsid=%d\n",
@@ -239,8 +246,9 @@ void ExMlcCommandChannel::handleMsg(ExMsg *pMsg) {
 	   case MSG_COMMAND_REPLY_TIMEOUT:
 		if (!pCommandReplyTimer->isCancelled()) {
 			LOG_ERROR(cEXTP,0,cCausePeriphError,
-				"Reply timer popped on port=%d, count=%d!\n",
-				port,pCommandReplyTimer->getCount());
+				"Command reply timeout on port=%d, count=%d, "
+				"lastRequest=0x%2.2X!\n",port,
+				pCommandReplyTimer->getCount(),lastRequest);
 			pMgr->exClose(ExMlcTransport::REASON_REPLY_TIMEOUT);
 		}
 		pCommandReplyTimer->setMsg(pCommandReplyTimeoutMsg);
@@ -274,7 +282,7 @@ void ExMlcCommandChannel::getEmptyBuffer(ExBdr **ppBdr,CommandUnion **pData) {
 	LOGD_ASSERT(ppBdr && pData,cEXTP,0,cCauseBadParm,"");
 	if (isClosing()) {
 		*ppBdr=0;
-		LOG_ERROR(cEXTP,0,cCauseBadState,
+		LOGD_ERROR(cEXTP,0,cCauseBadState,
 			"getEmptyBuffer(port=%d): closing!\n",
 			port);
 		return;
@@ -327,18 +335,22 @@ ExBdr *ExMlcCommandChannel::servicePullForwardData(void) {
 			ExBdr *_pBdr=pForwardRequestQueue->Peek();
 			if (_pBdr && forwardRequestCredit.grab()!=ERROR) {
 				pBdr=pForwardRequestQueue->Pop();
-				LOG_ASSERT(pBdr==_pBdr,
+				LOGD_ASSERT(pBdr==_pBdr,
 					cEXTP,0,cCauseBadParm,"");
-
-				/* If we pulled a request, then remember any
-				 * socket IDs in the command, because MLC
-				 * replies don't provide them. */
-				CommandUnion *data=bdrGetCommandUnion(pBdr);
-				lastPsid=data->options.psid;
-				lastSsid=data->options.ssid;
 			}
 		}
 	}
+	if (pBdr) {
+		/* If we pulled a request, then remember any
+		 * socket IDs in the command, because MLC
+		 * replies don't provide them. */
+		CommandUnion *data=bdrGetCommandUnion(pBdr);
+		if (!(data->request.command&REPLY_BIT)) {
+			lastRequest=data->request.command;
+			lastPsid=data->options.psid;
+			lastSsid=data->options.ssid;
+		}
+	}
 
 	return pBdr;
 }
@@ -384,7 +396,7 @@ int ExMlcCommandChannel::handlePiggybackCreditPart1(int credit,
 	if (pMlcTransport->revisionIsMlc()) {
 		/* Complain if explicit piggyback credit was provided. */
 		if (credit) {
-			LOG_ERROR(cEXTP,0,cCausePeriphError,
+			LOGD_ERROR(cEXTP,0,cCausePeriphError,
 				"handlePiggybackCreditPart1(port=%d,"
 				"channel=%d): "
 				"incorrect MLC implicit credit=%d!\n",
@@ -398,7 +410,7 @@ int ExMlcCommandChannel::handlePiggybackCreditPart1(int credit,
 	 * care if it granted more than we expected, except that we
 	 * won't use the excess for forward request credit. */
 	} else if (credit && !expectedCredit) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handlePiggybackCreditPart1(port=%d,channel=%d): "
 			"unexpected 1284.4 credit=%d!\n",
 			port,channel,credit);
@@ -423,7 +435,7 @@ int ExMlcCommandChannel::handlePiggybackCreditPart1(int credit,
 int ExMlcCommandChannel::handleForwardCreditPart1(int credit,
     int requestCredit) {
 	if (forwardRequestCredit.increment(requestCredit)==ERROR) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleForwardCreditPart1(port=%d,channel=%d,"
 			"credit=%d,requestCredit=%d): "
 			"request credit overflow!\n",
@@ -474,8 +486,8 @@ int ExMlcCommandChannel::grabReverseCredit(unsigned char *_data,int datalen) {
 	return ExMlcTransportChannel::grabReverseCredit(_data,datalen);
 }
 
-int ExMlcCommandChannel::grantReverseCredit(int max=0xFFFF,
-    unsigned char *_data=0,int datalen=0) {
+int ExMlcCommandChannel::grantReverseCredit(int max,
+    unsigned char *_data,int datalen) {
 	CommandUnion *data=(CommandUnion *)_data;
 
 	/* Don't check datalen, because that'll break on empty packets. */
@@ -645,6 +657,9 @@ void ExMlcCommandChannel::serviceReverseDataReceived(ExBdr *pBdr,int status,
 	 * returnBufferNotification so we don't grant credit. */
 	if (!data->options.consumeCredit) {
 		ExTransport::unmarkBDR(pBdr);
+#ifdef JD_DEBUGLITE
+		countReverseBufferReturns.increment();
+#endif
 	}
 
 	/* TODO: Assert that this isn't a chain? */
@@ -661,7 +676,7 @@ STATUS ExMlcCommandChannel::checkDatalen(CommandUnion *data,int datalen,
 		return OK;
 	}
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"checkDatalen(port=%d,command=0x%2.2X): "
 		"datalen=%d, expected=%d!\n",
 		port,data->request.command,datalen,expectedDatalen);
@@ -710,7 +725,7 @@ STATUS ExMlcCommandChannel::checkDatalen(CommandUnion *data,int datalen,
 
 #define REQUEST_LOOKUP_TCD(functionName,mlcError,dot4ReplyCall) \
 	if (!(tcd=pMlcTransport->lookupChannel(psid,ssid))) { \
-		LOG_ERROR(cEXTP,0,cCausePeriphError, \
+		LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 			functionName "(port=%d): " \
 			"invalid psid=%d, ssid=%d!\n", \
 			port,psid,ssid); \
@@ -724,7 +739,7 @@ STATUS ExMlcCommandChannel::checkDatalen(CommandUnion *data,int datalen,
 
 #define REPLY_LOOKUP_TCD(functionName) \
 	if (!(tcd=pMlcTransport->lookupChannel(psid,ssid))) { \
-		LOG_ERROR(cEXTP,0,cCausePeriphError, \
+		LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 			functionName "(port=%d): " \
 			"invalid psid=%d, ssid=%d!\n", \
 			port,psid,ssid); \
@@ -1045,7 +1060,7 @@ void ExMlcCommandChannel::handleCloseChannel(CommandUnion *data,int datalen) {
 
 	/* Don't allow the command channel to be closed. */
 	if (tcd==this) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleCloseChannel(port=%d): "
 			"peripheral tried to close command channel!\n",
 			port);
@@ -1370,7 +1385,7 @@ void ExMlcCommandChannel::handleCreditRequestReply(CommandUnion *data,
 
 void ExMlcCommandChannel::sendConfigSocket(int socketID,
     int maxPriToSecPacketSize,int maxSecToPriPacketSize,
-    int statusLevel=0) {
+    int statusLevel) {
 	DECLARE_VARIABLES;
 
 	GET_EMPTY_BUFFER(CMD_CONFIG_SOCKET);
@@ -1424,7 +1439,7 @@ void ExMlcCommandChannel::handleConfigSocket(CommandUnion *data,int datalen) {
 
 void ExMlcCommandChannel::sendConfigSocketReply(int result,int socketID,
     int maxPriToSecPacketSize,int maxSecToPriPacketSize,
-    int statusLevel=0) {
+    int statusLevel) {
 	DECLARE_VARIABLES;
 
 	GET_EMPTY_BUFFER(CMD_CONFIG_SOCKET_REPLY);
@@ -1683,7 +1698,7 @@ void ExMlcCommandChannel::handleGetServiceNameReply(CommandUnion *data,
 void ExMlcCommandChannel::mlcSendError(int error) {
 	DECLARE_VARIABLES;
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"sendError(port=%d,MLC): "
 		"error=0x%2.2X.\n",
 		port,error);
@@ -1711,7 +1726,7 @@ void ExMlcCommandChannel::mlcSendError(int error) {
 void ExMlcCommandChannel::dot4SendError(int psid,int ssid,int error) {
 	DECLARE_VARIABLES;
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"sendError(port=%d,1284.4): "
 		"psid=%d, ssid=%d, error=0x%2.2X.\n",
 		port,psid,ssid,error);
@@ -1759,7 +1774,7 @@ void ExMlcCommandChannel::handleError(CommandUnion *data,int datalen) {
 mlcError:
 		error=data->error.mlc.error;
 
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleError(port=%d,MLC): "
 			"error=0x%2.2X.\n",
 			port,error);
@@ -1772,7 +1787,7 @@ dot4Error:
 		ssid=data->error.dot4.ssid;
 		error=data->error.dot4.error;
 
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleError(port=%d,1284.4): "
 			"psid=%d, ssid=%d, error=0x%2.2X.\n",
 			port,psid,ssid,error);
@@ -1791,7 +1806,7 @@ void ExMlcCommandChannel::sendGenericReply(int command,int result) {
 
 		SET_DATALEN(LEN_REPLY);
 
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"sendGenericReply(port=%d): "
 			"command=0x%2.2X, result=%d.\n",
 			port,command,result);
