@@ -1,6 +1,6 @@
 // ExMlcTransport.cpp
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -66,7 +66,7 @@ const int ExMlcTransportChannel::DEFAULT_REVERSE_CREDIT_HEARTBEAT_DELAY;
 
 #define REV_REQUEST_VALIDATE_CHANNEL_OPEN(functionName,mlcError,dot4ReplyCall) \
 	if (!isOpen()) { \
-		LOG_ERROR(cEXTP,0,cCausePeriphError, \
+		LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 			functionName "(port=%d,channel=%d): not open!\n", \
 			port,channel); \
 		if (pMlcTransport->revisionIsMlc()) { \
@@ -79,7 +79,7 @@ const int ExMlcTransportChannel::DEFAULT_REVERSE_CREDIT_HEARTBEAT_DELAY;
 
 #define REV_REPLY_VALIDATE_LOCAL_REQUEST_PENDING(setFlag,functionName) \
 	if (!testFlagsSetClear(setFlag,0)) { \
-		LOG_ERROR(cEXTP,0,cCausePeriphError, \
+		LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 			functionName "(port=%d,channel=%d): unexpected!\n", \
 			port,channel); \
 		pCommandChannel->sendError(psid,ssid, \
@@ -91,7 +91,7 @@ const int ExMlcTransportChannel::DEFAULT_REVERSE_CREDIT_HEARTBEAT_DELAY;
 
 #define REV_REPLY_VALIDATE_SUCCESSFUL(functionName) \
 	if (result!=ExMlcTransport::RESULT_SUCCESS) { \
-		LOG_ERROR(cEXTP,0,cCausePeriphError, \
+		LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 			functionName "(port=%d,channel=%d): " \
 			"result=%d!\n", \
 			port,channel,result); \
@@ -102,7 +102,7 @@ const int ExMlcTransportChannel::DEFAULT_REVERSE_CREDIT_HEARTBEAT_DELAY;
 
 #define CREDIT_OVERFLOW(functionName) \
     do { \
-	LOG_ERROR(cEXTP,0,cCausePeriphError, \
+	LOGD_ERROR(cEXTP,0,cCausePeriphError, \
 		functionName "(port=%d,channel=%d): " \
 		"credit overflow!\n", \
 		port,channel); \
@@ -118,7 +118,7 @@ const int ExMlcTransportChannel::DEFAULT_REVERSE_CREDIT_HEARTBEAT_DELAY;
 
 ExMlcTransportChannel::ExMlcTransportChannel(ExMlcTransport *pMlcTransport,
     int channel,int localSocket,ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,
-    ExMlcCommandChannel *pCommandChannel,int final=1):
+    ExMlcCommandChannel *pCommandChannel,int final):
       ExTransportChannel(pMlcTransport,channel,localSocket,pMgr,
        pPhysicalPort,0) {
 	openingAnySetFlags|=FLAG_LOCAL_OPEN_PENDING;
@@ -545,7 +545,7 @@ sendOK:
 	if (pMlcTransport->remsockLookup(remoteSocket,&state,0,0)!=ERROR) {
 		if (state==ExMlcTransport::REMSOCK_STATE_CONFIGURED)
 			goto sendOK;
-		LOG_ASSERT(state==ExMlcTransport::REMSOCK_STATE_CONFIGURING,
+		LOGD_ASSERT(state==ExMlcTransport::REMSOCK_STATE_CONFIGURING,
 			cEXTP,0,cCauseBadState,"");
 		return;
 	}
@@ -573,7 +573,7 @@ void ExMlcTransportChannel::open_ts(int maxForwardDatalen,
 	if (pMlcTransport->revisionIsMlc()) {
 		status=pMlcTransport->remsockLookup(remoteSocket,&state,
 			&maxForwardPacketSize,&maxReversePacketSize);
-		LOG_ASSERT(status==OK &&
+		LOGD_ASSERT(status==OK &&
 			state==ExMlcTransport::REMSOCK_STATE_CONFIGURED,
 			cEXTP,0,cCauseBadState,"");
 		setDatalensFromPacketSizes();
@@ -631,7 +631,7 @@ void ExMlcTransportChannel::handleOpenChannelReply(int result,
 			status=S_hlio_BP_UNSUPPORTED_SOCKET;
 			break;
 		   default:
-			LOG_ERROR(cEXTP,0,cCausePeriphError,
+			LOGD_ERROR(cEXTP,0,cCausePeriphError,
 				"handleOpenChannelReply(port=%d,channel=%d): "
 				"result=%d!\n",port,channel,result);
 			int shouldWeDeactivate=0;
@@ -677,7 +677,7 @@ void ExMlcTransportChannel::setDatalensFromPacketSizes(void) {
 
 	/* Warn of invalid packet sizes (1<=x<=5, or both zero). */
 	if (!packetSizesAreValid(maxForwardPacketSize,maxReversePacketSize)) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"setDatalensFromPacketSizes(port=%d,channel=%d): "
 			"invalid maxForwardPacketSize=%d, "
 			"maxReversePacketSize=%d!\n",
@@ -941,7 +941,7 @@ void ExMlcTransportChannel::scheduleCreditRequest(SourceEnum source) {
 		break;
 
 	   default:
-		LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+		LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 	}
 }
 
@@ -982,7 +982,7 @@ void ExMlcTransportChannel::handleCredit(int psid,int ssid,int credit,
 
 	int part1Result=handleForwardCreditPart1(credit,cmdinfo);
 	if (part1Result==ERROR) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleCredit(port=%d,channel=%d): "
 			"credit overflow!\n",
 			port,channel);
@@ -1083,7 +1083,7 @@ void ExMlcTransportChannel::reverseDataReceived_ts(ExBdr *pHeaderBdr,
 
 	/* Make sure this channel is open. */
 	if (!isOpen()) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"reverseDataReceived_ts(port=%d,channel=%d): "
 			"not open!\n",port,channel);
 		mlcError=ExMlcTransport::MLC_ERROR_UNOPENED_CHANNEL_DATA;
@@ -1117,7 +1117,7 @@ void ExMlcTransportChannel::reverseDataReceived_ts(ExBdr *pHeaderBdr,
 
 	/* Make sure we have (or don't need) credit for this packet. */
 	if (grabReverseCredit(data,datalen)==ERROR) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"reverseDataReceived_ts(port=%d,channel=%d): "
 			"uncredited packet!\n",
 			port,channel);
@@ -1129,7 +1129,7 @@ void ExMlcTransportChannel::reverseDataReceived_ts(ExBdr *pHeaderBdr,
 	/* Handle piggyback credit.  Raise error if credit overflow. */
 	part1Result=handlePiggybackCreditPart1(header->credit,data,datalen,0);
 	if (part1Result==ERROR) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"reverseDataReceived_ts(port=%d,channel=%d): "
 			"piggyback credit overflow!\n",
 			port,channel);
@@ -1150,9 +1150,6 @@ void ExMlcTransportChannel::reverseDataReceived_ts(ExBdr *pHeaderBdr,
 		goto abort;
 	}
 
-	/* Copy control field from header into BDR. */
-	pDataBdr->setTransportHeaderFlags(header->control);
-
 	/* Send empty packets on up the stack. */
 	if (!pDataBdr) {
 		pDataBdr=pHeaderBdr;
@@ -1165,6 +1162,9 @@ void ExMlcTransportChannel::reverseDataReceived_ts(ExBdr *pHeaderBdr,
 		bdrCount--;
 	}
 
+	/* Copy control field from header into BDR. */
+	pDataBdr->setTransportHeaderFlags(header->control);
+
 	/* Credit back forfeited and header buffers. */
 	creditReturnedBuffers(reverseBuffersPerPacket-bdrCount);
 
@@ -1216,7 +1216,7 @@ void ExMlcTransportChannel::returnBufferNotification(ExBdr *pBdr) {
 }
 
 /* TODO: Add asserts in various places to validate credit management. */
-void ExMlcTransportChannel::creditReturnedBuffers(int returnedBuffers=0) {
+void ExMlcTransportChannel::creditReturnedBuffers(int returnedBuffers) {
 	uncreditedBuffers+=returnedBuffers;
 	if ((!isOpen() || isClosing()) && !isOpening()) {
 		LOG_INFO(cEXTP,0,
@@ -1244,8 +1244,8 @@ void ExMlcTransportChannel::uncreditCreditedBuffers(void) {
 		port,channel,credit,buffers,uncreditedBuffers);
 }
 
-int ExMlcTransportChannel::grantReverseCredit(int max=0xFFFF,
-    unsigned char *data=0,int datalen=0) {
+int ExMlcTransportChannel::grantReverseCredit(int max,
+    unsigned char *data,int datalen) {
 	int credit=reverseCreditToGrant;
 	if (credit>max) credit=max;
 	reverseCreditToGrant-=credit;
@@ -1324,7 +1324,7 @@ void ExMlcTransportChannel::scheduleCredit(SourceEnum source) {
 		break;
 
 	   default:
-		LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+		LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 	}
 
 	LOG_INFO(cEXTP,0,"scheduleCredit(port=%d,channel=%d,source=%d): "
@@ -1483,7 +1483,7 @@ const int ExMlcTransport::DEFAULT_MAX_REMOTE_SOCKETS;
 \*---------------------------------------------------------------------------*/
 
 ExMlcTransport::ExMlcTransport(ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,
-    int forwardTransactionCount,int channelCount,int final=1):
+    int forwardTransactionCount,int channelCount,int final):
       ExTransport(pMgr,pPhysicalPort,forwardTransactionCount,channelCount+1,0) {
 
 	/* For now, we will leave this at the default of zero. */
@@ -1506,7 +1506,7 @@ ExMlcTransport::ExMlcTransport(ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,
 	maxRemoteSockets=DEFAULT_MAX_REMOTE_SOCKETS;
 	exTknobGetInit(EX_KNOB_MLC_INIT_MAX_REMOTE_SOCKETS,&maxRemoteSockets);
 	remoteSocketArray=new RemoteSocket[maxRemoteSockets];
-	LOG_ASSERT(remoteSocketArray,cEXTP,0,cCauseNoMem,"");
+	LOGD_ASSERT(remoteSocketArray,cEXTP,0,cCauseNoMem,"");
 
 	if (final) {
 		reset(RESET_STARTUP);
@@ -1641,9 +1641,11 @@ void ExMlcTransport::dump(void) {
 void ExMlcTransport::transportResetComplete_ts(void) {
 	TCD tcd;
 	int timeout=DEFAULT_MLC_FORWARD_DATA_TIMEOUT;
+	absoluteMaxForwardDatalen-=LEN_HEADER;
+	absoluteMaxReverseDatalen-=LEN_HEADER;
 
 	tcd=allocateChannel(0,0,0,0,0);
-	LOG_ASSERT(tcd==pCommandChannel,cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(tcd==pCommandChannel,cEXTP,0,cCauseBadState,"");
 
 	tryMlc=DEFAULT_TRY_REVISION_MLC;
 	tknobGetWorkingValue(port,EX_KNOB_MLC_TRY_REVISION_MLC,&tryMlc);
@@ -1672,7 +1674,7 @@ void ExMlcTransport::sendInit(int nextRevision) {
 	   case REVISION_MLC:
 		goto skipMlc;
 	   default:
-		LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,
+		LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,
 			"sendInit(port=%d): "
 			"invalid requestedRevision=0x%2.2X!\n",
 			port,requestedRevision);
@@ -1697,7 +1699,7 @@ skipPreDot4:
 	}
 skipMlc:
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"sendInit(port=%d): exhausted all possible revisions!\n",
 		port);
 	pMgr->exClose(REASON_INIT_REVISION_REFUSED);
@@ -1710,7 +1712,7 @@ send:
 STATUS ExMlcTransport::handleInitReply(int result,int revision) {
 	/* Complain about but ignore second InitReply. */
 	if (revisionIsSet()) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleInitReply(port=%d): "
 			"ignoring extra InitReply!\n",
 			port);
@@ -1719,7 +1721,7 @@ STATUS ExMlcTransport::handleInitReply(int result,int revision) {
 
 	/* Complain but ignore if Init hasn't been sent yet. */
 	if (requestedRevision==REVISION_NONE) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleInitReply(port=%d): "
 			"Init hasn't been sent yet!\n",
 			port);
@@ -1735,7 +1737,7 @@ STATUS ExMlcTransport::handleInitReply(int result,int revision) {
 	}
 	/* Otherwise, refuse to do business with this peripheral. */
 	if (result!=RESULT_SUCCESS) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleInitReply(port=%d): result=%d!\n",
 			port,result);
 		pMgr->exClose(REASON_RECEIVED_UNSUCCESSFUL_REPLY);
@@ -1744,7 +1746,7 @@ STATUS ExMlcTransport::handleInitReply(int result,int revision) {
 
 	/* Complain about successful InitReply for wrong revision. */
 	if (revision!=requestedRevision) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleInitReply(port=%d,revision=0x%2.2X): "
 			"requested revision=0x%2.2X!\n",
 			port,revision,requestedRevision);
@@ -1828,7 +1830,7 @@ void ExMlcTransport::lookupRemoteSocket_ts(ExLookup *pLookupRequest) {
 		break;
 
 	   default:
-		LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+		LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 	}
 }
 
@@ -1909,7 +1911,7 @@ void ExMlcTransport::handleLookupReply(int lastSet,int result,
 
 #ifdef JD_DEBUGLITE
 	if (msg) {
-		LOG_ERROR(cEXTP,0,cCausePeriphError,
+		LOGD_ERROR(cEXTP,0,cCausePeriphError,
 			"handleLookupReply(port=%d,lastSet=%d,result=%d,"
 			"socketID=%d,serviceName=<%s>): %s!\n",
 			port,lastSet,result,socketID,serviceName,msg);
@@ -1954,7 +1956,7 @@ void ExMlcTransport::remsockReset(void) {
 
 STATUS ExMlcTransport::remsockLookup(int socketID,int *pState,
     int *pMaxForwardPacketSize,int *pMaxReversePacketSize) {
-	LOG_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
 
 	int i;
 
@@ -1976,7 +1978,7 @@ STATUS ExMlcTransport::remsockLookup(int socketID,int *pState,
 }
 
 STATUS ExMlcTransport::remsockAdd(int socketID) {
-	LOG_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
 
 	int i,firstFree=-1;
 
@@ -1984,7 +1986,7 @@ STATUS ExMlcTransport::remsockAdd(int socketID) {
 		if (remoteSocketArray[i].state==REMSOCK_STATE_UNCONFIGURED) {
 			if (firstFree<0) firstFree=i;
 		} else if (remoteSocketArray[i].socketID==socketID) {
-			LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,
+			LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,
 				"remsockAdd(port=%d,socketID=%d): "
 				"already in table!\n",
 				port,socketID);
@@ -1993,7 +1995,7 @@ STATUS ExMlcTransport::remsockAdd(int socketID) {
 	}
 
 	if (firstFree<0) {
-		LOG_ERROR(cEXTP,0,cCauseNoHandles,
+		LOGD_ERROR(cEXTP,0,cCauseNoHandles,
 			"remsockAdd(port=%d,socketID=%d): "
 			"table is full!\n",
 			port,socketID);
@@ -2011,7 +2013,7 @@ STATUS ExMlcTransport::remsockAdd(int socketID) {
 void ExMlcTransport::handleConfigSocketReply(int result,int socketID,
     int maxPriToSecPacketSize,int maxSecToPriPacketSize,
     int statusLevel) {
-	LOG_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
 
 	int status=OK;
 	int maxForwardPacketSize=
@@ -2038,7 +2040,7 @@ void ExMlcTransport::handleConfigSocketReply(int result,int socketID,
 
 STATUS ExMlcTransport::remsockUpdate(int socketID,
     int maxForwardPacketSize,int maxReversePacketSize) {
-	LOG_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
 
 	int i;
 
@@ -2047,7 +2049,7 @@ STATUS ExMlcTransport::remsockUpdate(int socketID,
 		    remoteSocketArray[i].socketID==socketID) {
 			if (remoteSocketArray[i].state!=
 			    REMSOCK_STATE_CONFIGURING) {
-				LOG_ERROR(cEXTP,0,cCausePeriphError,
+				LOGD_ERROR(cEXTP,0,cCausePeriphError,
 					"remsockUpdate(port=%d,socketID=%d): "
 					"already configured!\n",
 					port,socketID);
@@ -2063,14 +2065,14 @@ STATUS ExMlcTransport::remsockUpdate(int socketID,
 		}
 	}
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"remsockUpdate(port=%d,socketID=%d): not in table!\n",
 		port,socketID);
 	return ERROR;
 }
 
 STATUS ExMlcTransport::remsockDelete(int socketID) {
-	LOG_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(revisionIsMlc(),cEXTP,0,cCauseBadState,"");
 
 	int i;
 
@@ -2079,7 +2081,7 @@ STATUS ExMlcTransport::remsockDelete(int socketID) {
 		    remoteSocketArray[i].socketID==socketID) {
 			if (remoteSocketArray[i].state!=
 			    REMSOCK_STATE_CONFIGURING) {
-				LOG_ERROR(cEXTP,0,cCausePeriphError,
+				LOGD_ERROR(cEXTP,0,cCausePeriphError,
 					"remsockDelete(port=%d,socketID=%d): "
 					"already configured!\n",
 					port,socketID);
@@ -2093,7 +2095,7 @@ STATUS ExMlcTransport::remsockDelete(int socketID) {
 		}
 	}
 
-	LOG_ERROR(cEXTP,0,cCausePeriphError,
+	LOGD_ERROR(cEXTP,0,cCausePeriphError,
 		"remsockDelete(port=%d,socketID=%d): not in table!\n",
 		port,socketID);
 	return ERROR;
