@@ -1,6 +1,6 @@
 // ExMlcTransport.h
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,13 @@
 #ifndef EXMLCTRANSPORT_H
 #define EXMLCTRANSPORT_H
 
+#ifdef EX_TRANSPORT_UNIX_PORT
+#include <ExTransport.h>
+#else
 #include <bp/ex/transport/ExTransport.h>
+#endif
+
+#define ATTRIBUTE_PACKED __attribute__((packed))
 
 class ExMlcTransport;
 class ExMlcCommandChannel;
@@ -297,24 +303,24 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 
 	struct CmdRequest {
 		unsigned char command;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_REQUEST			=0x01;
 	struct CmdReply {
 		unsigned char command;
 		unsigned char result;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_REPLY			=0x02;
 
 	struct CmdInit {
 		unsigned char command;
 		unsigned char revision;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_INIT			=0x02;
 	struct CmdInitReply {
 		unsigned char command;
 		unsigned char result;
 		unsigned char revision;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_INIT_REPLY			=0x03;
 
 	struct CmdOpenChannel {
@@ -324,14 +330,14 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		union {
 		    struct {
 			unsigned char credit[2];
-		    } mlc;
+		    } mlc ATTRIBUTE_PACKED;
 		    struct {
 			unsigned char maxPriToSecPacketSize[2];
 			unsigned char maxSecToPriPacketSize[2];
 			unsigned char maxOutstandingCredit[2];
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_OPEN_CHANNEL		=0x05;
 	static const int DOT4_LEN_OPEN_CHANNEL		=0x09;
 	struct CmdOpenChannelReply {
@@ -340,7 +346,7 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		union {
 		    struct {
 			unsigned char credit[2];
-		    } mlc;
+		    } mlc ATTRIBUTE_PACKED;
 		    struct {
 			unsigned char psid;
 			unsigned char ssid;
@@ -348,9 +354,9 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 			unsigned char maxSecToPriPacketSize[2];
 			unsigned char maxOutstandingCredit[2];
 			unsigned char credit[2];
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_OPEN_CHANNEL_REPLY	=0x04;
 	static const int DOT4_LEN_OPEN_CHANNEL_REPLY	=0x0C;
 
@@ -358,7 +364,7 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		unsigned char command;
 		unsigned char psid;
 		unsigned char ssid;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_CLOSE_CHANNEL		=0x03;
 	struct CmdCloseChannelReply {
 		unsigned char command;
@@ -367,9 +373,9 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		    struct {
 			unsigned char psid;
 			unsigned char ssid;
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_CLOSE_CHANNEL_REPLY	=0x02;
 	static const int DOT4_LEN_CLOSE_CHANNEL_REPLY	=0x04;
 
@@ -378,7 +384,7 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		unsigned char psid;
 		unsigned char ssid;
 		unsigned char credit[2];
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_CREDIT			=0x05;
 	struct CmdCreditReply {
 		unsigned char command;
@@ -387,9 +393,9 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		    struct {
 			unsigned char psid;
 			unsigned char ssid;
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_CREDIT_REPLY		=0x02;
 	static const int DOT4_LEN_CREDIT_REPLY		=0x04;
 
@@ -400,12 +406,12 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		union {
 		    struct {
 			unsigned char creditRequested[2];
-		    } mlc;
+		    } mlc ATTRIBUTE_PACKED;
 		    struct {
 			unsigned char maxOutstandingCredit[2];
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int LEN_CREDIT_REQUEST		=0x05;
 	struct CmdCreditRequestReply {
 		unsigned char command;
@@ -413,14 +419,14 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		union {
 		    struct {
 			unsigned char credit[2];
-		    } mlc;
+		    } mlc ATTRIBUTE_PACKED;
 		    struct {
 			unsigned char psid;
 			unsigned char ssid;
 			unsigned char credit[2];
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_CREDIT_REQUEST_REPLY	=0x04;
 	static const int DOT4_LEN_CREDIT_REQUEST_REPLY	=0x06;
 
@@ -430,7 +436,7 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		unsigned char maxPriToSecPacketSize[2];
 		unsigned char maxSecToPriPacketSize[2];
 		unsigned char statusLevel;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_CONFIG_SOCKET		=0x07;
 	struct CmdConfigSocketReply {
 		unsigned char command;
@@ -438,43 +444,43 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		unsigned char maxPriToSecPacketSize[2];
 		unsigned char maxSecToPriPacketSize[2];
 		unsigned char statusLevel;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_CONFIG_SOCKET_REPLY	=0x07;
 
 	struct CmdExit {
 		unsigned char command;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_EXIT			=0x01;
 	struct CmdExitReply {
 		unsigned char command;
 		unsigned char result;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_EXIT_REPLY			=0x02;
 
 	struct CmdGetSocketID {
 		unsigned char command;
 		/*unsigned*/ char serviceName[MAX_SERVICE_NAME_LENGTH+1];
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_GET_SOCKET_ID		=0x01;
 	struct CmdGetSocketIDReply {
 		unsigned char command;
 		unsigned char result;
 		unsigned char socketID;
 		/*unsigned*/ char serviceName[MAX_SERVICE_NAME_LENGTH+1];
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_GET_SOCKET_ID_REPLY	=0x03;
 
 	struct CmdGetServiceName {
 		unsigned char command;
 		unsigned char socketID;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_GET_SERVICE_NAME		=0x02;
 	struct CmdGetServiceNameReply {
 		unsigned char command;
 		unsigned char result;
 		unsigned char socketID;
 		/*unsigned*/ char serviceName[MAX_SERVICE_NAME_LENGTH+1];
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_GET_SERVICE_NAME_REPLY	 =0x03;
 
 	struct CmdError {
@@ -482,14 +488,14 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 		union {
 		    struct {
 			unsigned char error;
-		    } mlc;
+		    } mlc ATTRIBUTE_PACKED;
 		    struct {
 			unsigned char psid;
 			unsigned char ssid;
 			unsigned char error;
-		    } dot4;
-		};
-	};
+		    } dot4 ATTRIBUTE_PACKED;
+		} ATTRIBUTE_PACKED;
+	} ATTRIBUTE_PACKED;
 	static const int MLC_LEN_ERROR			=0x02;
 	static const int DOT4_LEN_ERROR			=0x04;
 
@@ -541,6 +547,7 @@ class ExMlcCommandChannel: public ExMlcTransportChannel {
 	ExMlcCreditCounter forwardRequestCredit;
 
 	int allowErrorPackets;
+	int lastRequest;
 	int lastPsid;
 	int lastSsid;
 
@@ -741,7 +748,7 @@ class ExMlcTransport: public ExTransport {
 		unsigned char length[2];
 		unsigned char credit;
 		unsigned char control;
-	};
+	} ATTRIBUTE_PACKED;
 	static const int LEN_HEADER			=0x06;
 
 	static const int REMSOCK_STATE_UNCONFIGURED=0;
