@@ -1,6 +1,6 @@
 // ExTransport.cpp
 
-/* Copyright (C) 2000-2001 Hewlett-Packard Company
+/* Copyright (C) 2000-2003 Hewlett-Packard Company
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -93,7 +93,7 @@ void ExLookup__dump(ExLookup *_this) {
 \*---------------------------------------------------------------------------*/
 
 int ExLookup::getServiceName(char *dest,int maxlen) {
-	LOG_ASSERT(dest && maxlen>0,cEXTP,0,cCauseBadParm,
+	LOGD_ASSERT(dest && maxlen>0,cEXTP,0,cCauseBadParm,
 		"ExLookup::getServiceName: "
 		"invalid dest=0x%8.8X or maxlen=%d!\n",
 		dest,maxlen);
@@ -102,7 +102,7 @@ int ExLookup::getServiceName(char *dest,int maxlen) {
 	strncpy(dest,serviceName,maxlen);
 	if (dest[maxlen-1]) {
 		dest[maxlen-1]=0;
-		LOG_ERROR(cEXTP,0,cCauseNoMem,
+		LOGD_ERROR(cEXTP,0,cCauseNoMem,
 			"ExLookup::getServiceName: truncated string!\n");
 		return maxlen;
 	}
@@ -111,12 +111,12 @@ int ExLookup::getServiceName(char *dest,int maxlen) {
 }
 
 int ExLookup::setServiceName(char *src,int len,int preserveLookupType) {
-	LOG_ASSERT(src,cEXTP,0,cCauseBadParm,
+	LOGD_ASSERT(src,cEXTP,0,cCauseBadParm,
 		"ExLookup::setServiceName: invalid src=0x%8.8X!\n",src);
 
 	if (!len) len=strlen(src);
 	if (len>MAX_SERVICE_NAME_LEN) {
-		LOG_ERROR(cEXTP,0,cCauseNoMem,
+		LOGD_ERROR(cEXTP,0,cCauseNoMem,
 			"ExLookup::setServiceName: truncated string!\n");
 		len=MAX_SERVICE_NAME_LEN;
 	}
@@ -150,7 +150,7 @@ const int ExTransportChannel::closedClearFlags;
 \*---------------------------------------------------------------------------*/
 
 ExTransportChannel::ExTransportChannel(ExTransport *pTransport,int channel,
-    int localSocket,ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,int final=1) {
+    int localSocket,ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,int final) {
 	port=pMgr->getPortNumber();
 
 	SETTHIS(channel);
@@ -382,7 +382,7 @@ void ExTransportChannel::dump(void) {
 #endif
 
 void ExTransportChannel::handleMsg(ExMsg *pMsg) {
-	LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+	LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 }
 
 /*---------------------------------------------------------------------------*\
@@ -434,6 +434,7 @@ void ExTransportChannel::setRemoteSocket(int remoteSocket,
 	LOG_ASSERT(isClosed(),cEXTP,0,cCauseBadState,"");
 
 	SETTHIS(remoteSocket);
+	adjustForAbsoluteMaxDatalens(&maxForwardDatalen,&maxReverseDatalen);
 	// There's no need to set these here:
 	// SETTHIS(maxForwardDatalen);
 	// SETTHIS(maxReverseDatalen);
@@ -468,9 +469,10 @@ void ExTransportChannel::open(int maxForwardDatalen,int maxReverseDatalen) {
 		port,channel,maxForwardDatalen,maxReverseDatalen);
 
 	LOG_ASSERT(isClosed(),cEXTP,0,cCauseBadState,"");
-	LOG_ASSERT(getRemoteSocket()!=ERROR,cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(getRemoteSocket()!=ERROR,cEXTP,0,cCauseBadState,"");
 
 	reset(RESET_OPEN);
+	adjustForAbsoluteMaxDatalens(&maxForwardDatalen,&maxReverseDatalen);
 
 	open_ts(maxForwardDatalen,maxReverseDatalen);
 }
@@ -499,10 +501,23 @@ void ExTransportChannel::sendChannelOpenResponse(int status) {
 |* Close:
 \*---------------------------------------------------------------------------*/
 
-void ExTransportChannel::close(void) {
-	LOG_ENTRY(cEXTP,0,"close(port=%d,channel=%d)\n",port,channel);
+void ExTransportChannel::close(int flushForwardData) {
+	LOG_ENTRY(cEXTP,0,"close(port=%d,channel=%d,flushForwardData=%d)\n",
+		port,channel,flushForwardData);
 
-	LOG_ASSERT(isOpen(),cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(isOpen(),cEXTP,0,cCauseBadState,"");
+
+	if (flushForwardData) {
+		if (currentForwardBuffer) {
+			serviceForwardDataResponse(currentForwardBuffer,
+				S_hlio_BP_CHAN_CLOSING);
+			currentForwardBuffer=0;
+		}
+		if (currentGrabbedTransaction!=ERROR) {
+			pTransport->returnForwardTransaction();
+			currentGrabbedTransaction=ERROR;
+		}
+	}
 
 	close_ts();
 }
@@ -524,7 +539,7 @@ void ExTransportChannel::sendChannelCloseResponse(int status) {
 |* BDR management for forward/reverse data flow:
 \*---------------------------------------------------------------------------*/
 
-void ExTransportChannel::markBDR(ExBdr *pBdr,ExCounter *pCounter=0) {
+void ExTransportChannel::markBDR(ExBdr *pBdr,ExCounter *pCounter) {
 	while (pBdr) {
 #ifdef JD_DEBUGLITE
 		if (pCounter) pCounter->increment();
@@ -613,6 +628,7 @@ void ExTransportChannel::forwardDataAvailable(void) {
 		/* CAUTION -- Don't block the channel after this point,
 		 * because grabForwardTransaction() starts a timer! */
 
+		currentForwardBuffer->setIsFirstDataBdr(1);
 		if (addOverhead(&currentForwardBuffer)==ERROR) {
 			LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,
 				"forwardDataAvailable(port=%d,channel=%d): "
@@ -663,6 +679,7 @@ void ExTransportChannel::forwardDataResponse(ExBdr *pBdr,int status) {
 		"pBdr=0x%8.8X,status=%d)\n",port,channel,pBdr,status);
 
 	removeOverhead(&pBdr);
+	currentForwardBuffer->setIsFirstDataBdr(0);
 
 	serviceForwardDataResponse(pBdr,status);
 
@@ -739,7 +756,7 @@ void ExTransport__nolog(void) {
 }
 
 ExTransport::ExTransport(ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,
-    int forwardTransactionCount,int channelCount,int final=1):
+    int forwardTransactionCount,int channelCount,int final):
       forwardTransactionCounter(forwardTransactionCount) {
 	if (!ExTransportRegisteredWithLoggingManager) {
 #ifdef JD_DEBUGLITE
@@ -788,18 +805,18 @@ ExTransport::ExTransport(ExMgr *pMgr,ExPhysicalPort *pPhysicalPort,
 }
 
 void ExTransport::initChannelArray(void) {
-	LOG_ASSERT(channelCount>0,cEXTP,0,cCauseBadParm,
+	LOGD_ASSERT(channelCount>0,cEXTP,0,cCauseBadParm,
 		"initChannelArray(port=%d): invalid channelCount=%d!\n",
 		port,channelCount);
-	LOG_ASSERT(!channelArray,cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(!channelArray,cEXTP,0,cCauseBadState,"");
 	channelArray=new TCD[channelCount];
-	LOG_ASSERT(channelArray,cEXTP,0,cCauseNoMem,
+	LOGD_ASSERT(channelArray,cEXTP,0,cCauseNoMem,
 		"ExTransport(port=%d): failed to malloc channelArray!\n",
 		port);
 
 	for (int channel=0;channel<channelCount;channel++) {
 		channelArray[channel]=newChannel(channel);
-		LOG_ASSERT(channelArray[channel],cEXTP,0,cCauseNoMem,
+		LOGD_ASSERT(channelArray[channel],cEXTP,0,cCauseNoMem,
 			"initChannelArray(port=%d): "
 			"failed to malloc channel=%d!\n",
 			port,channel);
@@ -836,6 +853,8 @@ void ExTransport::reset(int rtype) {
 		reverseDataBufferSize=0;
 		maxForwardBdrsPerTransaction=0;
 		nextChannelToAllocate=0;
+		absoluteMaxForwardDatalen=0;
+		absoluteMaxForwardDatalen=0;
 
 		clearBlockedChannel();
 
@@ -886,6 +905,14 @@ void ExTransport::registerStuff(char *title) {
 		&nextChannelToAllocate,
 		"%s[%d].nextChannelToAllocate",
 		title,port);
+	nettestRegister(NETTEST_TYPE_INT32,
+		&absoluteMaxForwardDatalen,
+		"%s[%d].absoluteMaxForwardDatalen",
+		title,port);
+	nettestRegister(NETTEST_TYPE_INT32,
+		&absoluteMaxReverseDatalen,
+		"%s[%d].absoluteMaxReverseDatalen",
+		title,port);
 	nettestRegister(NETTEST_TYPE_INT32,
 		&nextBlockedChannel,
 		"%s[%d].nextBlockedChannel",
@@ -931,6 +958,10 @@ void ExTransport::dump(void) {
 		maxForwardBdrsPerTransaction);
 	printf("nextChannelToAllocate=%d\n",
 		nextChannelToAllocate);
+	printf("absoluteMaxForwardDatalen=%d\n",
+		absoluteMaxForwardDatalen);
+	printf("absoluteMaxReverseDatalen=%d\n",
+		absoluteMaxReverseDatalen);
 	printf("nextBlockedChannel=%d\n",
 		nextBlockedChannel);
     int depth=lookupQueue.depth();
@@ -1022,7 +1053,7 @@ void ExTransport::handleMsg(ExMsg *pMsg) {
 		break;
 
 	   default:
-		LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+		LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 	}
 }
 
@@ -1049,13 +1080,16 @@ void ExTransport::activate(int reverseDataBufferCount,
 	pPhysicalPort->resetTransport(this);
 }
 
-void ExTransport::transportResetComplete(void) {
+void ExTransport::transportResetComplete(int absoluteMaxForwardDatalen,
+    int absoluteMaxReverseDatalen) {
 	int timeout=DEFAULT_FORWARD_DATA_TIMEOUT;
-
 	tknobGetWorkingValue(port,EX_KNOB_TRANSPORT_FORWARD_DATA_TIMEOUT,
 		&timeout);
 	pForwardDataTimer->setDelay(timeout);
 
+	SETTHIS(absoluteMaxForwardDatalen);
+	SETTHIS(absoluteMaxReverseDatalen);
+
 	transportResetComplete_ts();
 }
 
@@ -1075,7 +1109,7 @@ TCD ExTransport::allocateChannel(ExService *pService,SCD scd,
     int forwardDataPriority,int minBuffersRequired,int benefitOfMoreBuffers) {
 	TCD tcd;
 
-	LOG_ASSERT(nextChannelToAllocate<channelCount,
+	LOGD_ASSERT(nextChannelToAllocate<channelCount,
 		cEXTP,0,cCauseNoHandles,
 		"allocateChannel(port=%d): "
 		"no available channels!\n",port);
@@ -1141,7 +1175,7 @@ void ExTransport::initialRequestsComplete(void) {
 
 	/* Calculate number of extra buffers available. */
 	extraBuffers=reverseDataBufferCount-accumGrantedBuffers-accumMinBuffers;
-	LOG_ASSERT(extraBuffers>=0,cEXTP,0,cCauseBadParm,
+	LOGD_ASSERT(extraBuffers>=0,cEXTP,0,cCauseBadParm,
 		"waitingForInitialRequests(port=%d): extraBuffers=%d #1!\n",
 		port,extraBuffers);
 
@@ -1166,7 +1200,7 @@ void ExTransport::initialRequestsComplete(void) {
 
 	/* Make sure we didn't give out too many buffers. */
 	extraBuffers=reverseDataBufferCount-accumGrantedBuffers;
-	LOG_ASSERT(extraBuffers>=0,cEXTP,0,cCauseBadParm,
+	LOGD_ASSERT(extraBuffers>=0,cEXTP,0,cCauseBadParm,
 		"waitingForInitialRequests(port=%d): extraBuffers=%d #2!\n",
 		port,extraBuffers);
 
@@ -1199,7 +1233,7 @@ void ExTransport::initialRequestsComplete(void) {
 #endif
 
 	/* Make sure we didn't give out too many buffers. */
-	LOG_ASSERT(accumGrantedBuffers<=reverseDataBufferCount,
+	LOGD_ASSERT(accumGrantedBuffers<=reverseDataBufferCount,
 		cEXTP,0,cCauseBadParm,
 		"waitingForInitialRequests(port=%d): "
 		"accumGrantedBuffers=%d > reverseDataBufferCount=%d!\n",
@@ -1302,7 +1336,7 @@ void ExTransport::lookupRemoteSocket_ts(ExLookup *_pLookupRequest) {
 	int i,status=S_hlio_BP_SERVICE_LOOKUP_FAILED;
 
 	ExLookup *pLookupRequest=lookupQueue.pop();
-	LOG_ASSERT(pLookupRequest==_pLookupRequest,cEXTP,0,cCauseBadParm,"");
+	LOGD_ASSERT(pLookupRequest==_pLookupRequest,cEXTP,0,cCauseBadParm,"");
 	int lastSet=pLookupRequest->getLastSet();
 	int socketID=pLookupRequest->getSocketID();
 	char *serviceName=pLookupRequest->getServiceName();
@@ -1328,7 +1362,7 @@ void ExTransport::lookupRemoteSocket_ts(ExLookup *_pLookupRequest) {
 			}
 
 		} else {
-			LOG_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
+			LOGD_ERROR_FATAL(cEXTP,0,cCauseBadParm,"");
 		}
 	}
 
@@ -1350,7 +1384,7 @@ void ExTransport::flushLookupQueue(void) {
 
 void ExTransport::sendRemoteSocketLookupResponse(ExLookup *pLookupRequest) {
 	ExService *pService=pLookupRequest->getService();
-	LOG_ASSERT(pService,cEXTP,0,cCauseBadParm,"");
+	LOGD_ASSERT(pService,cEXTP,0,cCauseBadParm,"");
 
 	LOG_ENTRY(cEXTP,0,
 		"sendRemoteSocketLookupResponse(port=%d,"
@@ -1454,7 +1488,7 @@ void ExTransport::unmarkBDR(ExBdr *pBdr) {
 
 ExTransportChannel *ExTransport::lookupChannel(ExBdr *pBdr) {
 	TCD tcd=channelArray[0];
-	LOG_ASSERT(tcd,cEXTP,0,cCauseBadState,"");
+	LOGD_ASSERT(tcd,cEXTP,0,cCauseBadState,"");
 	return tcd;
 }
 
